# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from unittest.mock import create_autospec
from superdesk.ftp import ftp_connect
from superdesk.tests import TestCase
import ftplib
from apps.publish import init_app
from superdesk.publish.transmitters.ftp import FTPPublishService
import io
from unittest import mock


ASSOCIATIONS = {
    'featuremedia': {
        'renditions': {
            '16-9': {
                'media': '5e448ee9016d1f63a92f040b',
                'width': 1280,
                'href': '/5e448ee9016d1f63a92f040b.jpg',
                'height': 720,
                'mimetype': 'image/jpeg',
                'poi': {'y': 1458, 'x': 1331},
            },
            '4-3': {
                'media': '5e448ee8016d1f63a92f0408',
                'width': 800,
                'href': '/5e448ee8016d1f63a92f0408.jpg',
                'height': 600,
                'mimetype': 'image/jpeg',
                'poi': {'y': 1502, 'x': 1162},
            },
            'original': {
                'media': '5e448e47016d1f63a92f03b8',
                'width': 5568,
                'href': '/5e448e47016d1f63a92f03b8.jpg',
                'height': 3712,
                'mimetype': 'image/jpeg',
                'poi': {'y': 1781, 'x': 1893},
            },
        }
    },
    'editor_0': {
        'priority': 6,
        'subject': [{'code': '02000000', 'name': 'crime, law and justice'}],
        'body_text': 'pic alt',
        'byline': 'The Great Unwashed',
        'renditions': {
            '16-9': {
                'media': '5e448dd1016d1f63a92f039e',
                'width': 1280,
                'href': '/5e448dd1016d1f63a92f039e.png',
                'height': 720,
                'mimetype': 'image/png',
            },
            '4-3': {
                'media': '5e448dd1016d1f63a92f0398',
                'width': 800,
                'href': '/5e448dd1016d1f63a92f0398.png',
                'height': 600,
                'mimetype': 'image/png',
            },
            'original': {
                'media': '5e448dd1016d1f63a92f0393',
                'width': 1615,
                'href': '/upload-raw/5e448dd1016d1f63a92f0393.png',
                'height': 1026,
                'mimetype': 'image/png',
            },
        },
    },
}


class NotFoundResponse():
    status_code = 404


class TestMedia(io.BytesIO):
    _id = 'media-id'
    filename = 'foo.txt'
    mimetype = 'text/plain'


def mockGet(self, _id, resource=None):
    return b'binary'


class FTPPublishServiceTestCase(TestCase):
    def setUp(self):
        init_app(self.app)

    item = {'item_id': 'abc',
            'format': 'NITF',
            'formatted_item': '1234567890'}

    def is_item_loaded(self, url, uploaded_filename):
        config = FTPPublishService().config_from_url(url)
        with ftplib.FTP(config.get('host')) as ftp:
            ftp.login(config.get('username'), config.get('password'))
            ftp.cwd(config.get('path', ''))
            ftp.set_pasv(config.get('passive', False))

            for filename, facts in ftp.mlsd():
                if filename == uploaded_filename:
                    return True
            return False

    def test_it_can_connect(self):
        service = FTPPublishService()

        if 'FTP_URL' not in os.environ:
            return

        config = service.config_from_url(os.environ['FTP_URL'])
        self.item['destination'] = {'config': config}

        self.assertEqual('test', config['path'])
        self.assertEqual('localhost', config['host'])

        service._transmit(self.item, subscriber={'config': config})
        self.assertTrue(self.is_item_loaded(config, 'abc.ntf'))

    @mock.patch('ftplib.FTP', autospec=True)
    @mock.patch('superdesk.storage.SuperdeskGridFSMediaStorage.get', mockGet)
    @mock.patch('superdesk.publish.transmitters.ftp.get_renditions_spec', return_value={'16-9': {}, '4-3': {}})
    def test_with_associations(self, mock_ftp_constructor, *args):
        item = {
            'associations': {
                'featuremedia': ASSOCIATIONS['featuremedia'],
            }}

        service = FTPPublishService()

        service._copy_published_media_files(item, mock_ftp_constructor)

        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448e47016d1f63a92f03b8.jpg', b'binary')
        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448ee8016d1f63a92f0408.jpg', b'binary')
        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448ee9016d1f63a92f040b.jpg', b'binary')

    @mock.patch('superdesk.ftp.ftplib.FTP', autospec=True)
    @mock.patch('superdesk.storage.SuperdeskGridFSMediaStorage.get', mockGet)
    @mock.patch('superdesk.publish.transmitters.ftp.get_renditions_spec', return_value={'16-9': {}, '4-3': {}})
    def test_with_association_and_embed(self, mock_ftp_constructor, *args):
        item = {'associations': ASSOCIATIONS}

        service = FTPPublishService()
        service._copy_published_media_files(item, mock_ftp_constructor)

        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448e47016d1f63a92f03b8.jpg', b'binary')
        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448ee8016d1f63a92f0408.jpg', b'binary')
        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448ee9016d1f63a92f040b.jpg', b'binary')
        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448dd1016d1f63a92f0393.png', b'binary')
        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448dd1016d1f63a92f0398.png', b'binary')
        mock_ftp_constructor.storbinary.assert_any_call('STOR 5e448dd1016d1f63a92f039e.png', b'binary')

    @mock.patch('superdesk.publish.transmitters.ftp.ftp_connect')
    @mock.patch('superdesk.storage.SuperdeskGridFSMediaStorage.get', mockGet)
    def test_publish_non_ninjs_item_assoc(self, ftp_connect_mock, *args):
        service = FTPPublishService()
        queue_item = {
            'item_id': 'someid',
            'item_version': 3,
            'formatted_item': '<?xml ...>',  # something json won't handle
            'destination': {'config': {'push_associated': True}},
        }

        ftp_mock = create_autospec(ftplib.FTP)()
        context_mock = mock.Mock()
        context_mock.__enter__ = mock.Mock(return_value=ftp_mock)
        context_mock.__exit__ = mock.Mock(return_value=None)
        ftp_connect_mock.return_value = context_mock

        self.app.data.insert('published', [
            {
                'item_id': 'someid',
                '_current_version': 3,
                'associations': ASSOCIATIONS,
            }
        ])

        subscriber = {}
        service._transmit(queue_item, subscriber)

        ftp_mock.storbinary.assert_any_call('STOR 5e448e47016d1f63a92f03b8.jpg', b'binary')
        ftp_mock.storbinary.assert_any_call('STOR 5e448dd1016d1f63a92f0393.png', b'binary')



import pytz
import warnings

from datetime import datetime
from unittest.mock import patch
from superdesk.tests import TestCase
from superdesk.io.feeding_services import FeedingService
from superdesk.errors import SuperdeskIngestError


class TestFeedingService(FeedingService):
    Name = 'test_feeding'

    def _update(self, provider, update):
        pass


class FeedingServiceParserTestCase(TestCase):

    def test_restricted_parser(self):
        feeding_service = TestFeedingService()
        with patch.dict('superdesk.io.feeding_services.restricted_feeding_service_parsers',
                        {'test_feeding': {'ninjs': True}}):
            with self.assertRaises(SuperdeskIngestError):
                feeding_service.config_test({'feed_parser': 'foo', 'feeding_service': 'test_feeding'})

    def test_add_timestamps_warning(self):
        item = {}
        feeding_service = TestFeedingService()
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            feeding_service.add_timestamps(item)
        assert len(w) == 1
        assert issubclass(w[0].category, DeprecationWarning)

    def test_localize_timestamps(self):
        item = {'firstcreated': datetime(2019, 10, 15, 10, 59, 0)}
        self.assertIsNone(item['firstcreated'].tzinfo)
        feeding_service = TestFeedingService()
        feeding_service.localize_timestamps(item)
        self.assertEqual(pytz.utc, item['firstcreated'].tzinfo)




# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os

import superdesk
from superdesk.io.feed_parsers.rfc822 import EMailRFC822FeedParser
from superdesk.tests import TestCase, setup
from superdesk.errors import IngestEmailError
from superdesk.users.services import UsersService


class RFC822TestCase(TestCase):
    filename = 'simple_email.txt'

    def setUp(self):
        setup(context=self)
        with self.app.app_context():
            # mock one user:
            user_service = UsersService(
                'users', backend=superdesk.get_backend())
            self.user_id = user_service.create([{
                'name': 'user',
                'user_type': 'administrator',
                'email': 'asender@a.com.au'
            }])[0]

            provider = {'name': 'Test'}
            dirname = os.path.dirname(os.path.realpath(__file__))
            fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
            with open(fixture, mode='rb') as f:
                bytes = f.read()
            parser = EMailRFC822FeedParser()
            self.items = parser.parse([(1, bytes)], provider)

    def test_headline(self):
        self.assertEqual(self.items[0]['headline'], 'Test message 1234')

    def test_body(self):
        self.assertEqual(self.items[0]['body_html'].strip(),
                         '<div dir="ltr">body text<br clear="all"/><div>&#13;\n</div></div>')

    def test_from(self):
        self.assertEqual(self.items[0]['original_source'],
                         'a sender <asender@a.com.au>')
        self.assertEqual(self.items[0]['original_creator'], self.user_id)


class RFC822ComplexTestCase(TestCase):
    filename = 'composite_email.txt'

    def setUp(self):
        setup(context=self)
        with self.app.app_context():
            provider = {'name': 'Test'}
            dirname = os.path.dirname(os.path.realpath(__file__))
            fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
            with open(fixture, mode='rb') as f:
                bytes = f.read()
            parser = EMailRFC822FeedParser()
            self.items = parser.parse([(1, bytes)], provider)

    def test_composite(self):
        self.assertEqual(len(self.items), 3)
        for item in self.items:
            self.assertIn('versioncreated', item)

    def test_from(self):
        self.assertEqual(self.items[0]['original_source'],
                         'someone <a@a.com.au>')
        self.assertNotIn('original_creator', self.items[0])


class RFC822OddCharSetTestCase(TestCase):
    filename = 'odd_charset_email.txt'

    def setUp(self):
        setup(context=self)
        with self.app.app_context():
            provider = {'name': 'Test'}
            dirname = os.path.dirname(os.path.realpath(__file__))
            fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
            with open(fixture, mode='rb') as f:
                bytes = f.read()
            parser = EMailRFC822FeedParser()
            self.items = parser.parse([(1, bytes)], provider)

    def test_headline(self):
        # This tests a subject that fails to decode but we just try a string conversion
        self.assertEqual(self.items[0]['headline'], '=?windows-1252?Q?TravTalk���s_Special_for_TAAI_convention?=')

    def test_body(self):
        self.assertRegex(self.items[0]['body_html'], '<span lang="EN-AU">')


class RFC822CharSetInSubjectTestCase(TestCase):
    filename = 'charset_in_subject_email.txt'

    def setUp(self):
        setup(context=self)
        with self.app.app_context():
            provider = {'name': 'Test'}
            dirname = os.path.dirname(os.path.realpath(__file__))
            fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
            with open(fixture, mode='rb') as f:
                bytes = f.read()
            parser = EMailRFC822FeedParser()
            self.items = parser.parse([(1, bytes)], provider)

    def test_headline(self):
        # This test a subject that has a charset that decodes correctly
        self.assertEqual(self.items[0]['headline'], 'Google Apps News March 2015')


class RFC822FormattedEmailTestCase(TestCase):

    def setUp(self):
        setup(context=self)
        with self.app.app_context():
            self.app.data.insert('users', [{
                '_id': 123,
                'name': 'user',
                'user_type': 'administrator',
                'email': 'eharvey@aap.com.au',
                'byline': 'E Harvey'
            }])
            self.app.data.insert('desks', [{'_id': 1, 'name': 'new zealand'}])
            self.app.data.insert('vocabularies', [{'_id': 'locators',
                                                  'items': [{'is_active': True,
                                                             'name': 'ADV', 'qcode': 'ADV', 'group': ''}]},
                                                  {'_id': 'priority', 'items': [{'is_active': True,
                                                                                 'name': 'Urgent', 'qcode': '3'}]}])

            self.provider = {'name': 'Test', 'config': {'formatted': True}}

    def test_parsed_values(self):
        filename = 'googleform.txt'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', filename))
        with open(fixture, mode='rb') as f:
            bytes = f.read()
        parser = EMailRFC822FeedParser()

        self.items = parser.parse([(1, bytes)], self.provider)
        self.assertEqual(self.items[0]['headline'], 'TEST NZ HEADER')
        self.assertEqual(self.items[0]['task']['desk'], 1)
        self.assertEqual(self.items[0]['original_creator'], 123)
        self.assertEqual(self.items[0]['urgency'], 1)
        self.assertEqual(self.items[0]['dateline']['text'], 'AUCKLAND, May 5 AAP -')
        self.assertEqual(self.items[0]['place'][0]['qcode'], 'ADV')
        self.assertEqual(self.items[0]['flags']['marked_for_legal'], True)
        self.assertEqual(self.items[0]['priority'], 3)
        self.assertEqual(self.items[0]['byline'], 'E Harvey')

    def test_parsed_values_Email_Address(self):
        filename = 'googleform1.txt'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', filename))
        with open(fixture, mode='rb') as f:
            bytes = f.read()
        parser = EMailRFC822FeedParser()
        self.items = parser.parse([(1, bytes)], self.provider)
        self.assertEqual(self.items[0]['headline'], 'Arnold \'worried\' about maroon-clad Roar')
        self.assertEqual(self.items[0]['task']['desk'], 1)
        self.assertEqual(self.items[0]['original_creator'], 123)
        self.assertEqual(self.items[0]['urgency'], 2)
        self.assertEqual(self.items[0]['dateline']['text'], 'BRISBANE, Nov 18 AAP -')
        self.assertEqual(self.items[0]['byline'], 'E Harvey')


class RFC822JsonEmailTestCase(TestCase):
    vocab = [{'_id': 'categories', 'items': [{'is_active': True, 'name': 'Domestic Sport', 'qcode': 's'}]}]
    desk = [{'_id': 1, 'name': 'Brisbane'}]
    user = [{'_id': 1, 'email': 'mock@mail.com.au', 'byline': 'A Mock Up', 'sign_off': 'TA'}]

    def setUp(self):
        self.app.data.insert('vocabularies', self.vocab)
        self.app.data.insert('desks', self.desk)
        self.app.data.insert('users', self.user)
        with self.app.app_context():
            self.provider = {'name': 'Test', 'config': {'formatted': True}}

    def test_formatted_email_parser(self):
        filename = 'json-email.txt'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.join(dirname, '../fixtures', filename)
        with open(fixture, mode='rb') as f:
            bytes = f.read()
        parser = EMailRFC822FeedParser()
        self.items = parser.parse([(1, bytes)], self.provider)

        self.assertEqual(self.items[0]['priority'], 5)
        self.assertEqual(self.items[0]['sign_off'], 'TA')
        self.assertEqual(self.items[0]['anpa_category'], [{'qcode': 's'}])
        self.assertEqual(self.items[0]['body_html'], '<p>Lorem ipsum</p>')
        self.assertEqual(self.items[0]['abstract'], 'Abstract-2')
        self.assertEqual(self.items[0]['headline'], 'Headline-2')
        self.assertEqual(self.items[0]['original_creator'], 1)
        self.assertEqual(self.items[0]['task']['desk'], 1)
        self.assertEqual(self.items[0]['urgency'], 4)
        self.assertEqual(self.items[0]['type'], 'text')
        self.assertEqual(self.items[0]['guid'], '<001a1140cd6ecd9de8051fab814d@google.com>')
        self.assertEqual(self.items[0]['original_source'], 'mock@mail.com.au')
        self.assertEqual(self.items[0]['slugline'], 'Slugline-2')
        self.assertEqual(self.items[0]['byline'], 'A Mock Up')

    def test_bad_user(self):
        filename = 'json-email-bad-user.txt'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.join(dirname, '../fixtures', filename)
        with open(fixture, mode='rb') as f:
            bytes = f.read()
        parser = EMailRFC822FeedParser()

        try:
            with self.assertRaises(IngestEmailError) as exc_context:
                self.items = parser.parse([(1, bytes)], self.provider)
        except Exception:
            self.fail('Expected exception type was not raised.')

        ex = exc_context.exception
        self.assertEqual(ex.code, 6001)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .volume_cubic_yard_to_metric import cubic_yard_to_metric


class VolumeTestCase(unittest.TestCase):

    def test(self):
        text = 'Total volume is 100.50 cubic yards for this land. '\
               'Total volume is 15.7 cubic yds for this land. '\
               'Total volume is 1 Cubic Yard for this land. '\
               'Total volume is 1-16 cu-yd for this land. '\
               'Total volume is 1-16 cb. yds for this land. '\
               'Total volume is 16.7-Cubic-yd for this land. '\
               'Total volume is 16,500-cu. yd. for this land. '\

        item = {'body_html': text}
        res, diff = cubic_yard_to_metric(item)
        self.assertEqual(diff['100.50 cubic yards'], '100.50 cubic yards (76.84 cubic meter)')
        self.assertEqual(diff['15.7 cubic yds'], '15.7 cubic yds (12.0 cubic meter)')
        self.assertEqual(diff['1 Cubic Yard'], '1 Cubic Yard (0.76 cubic meter)')
        self.assertEqual(diff['1-16 cu-yd'], '1-16 cu-yd (0.76-12 cubic meter)')
        self.assertEqual(diff['1-16 cb. yds'], '1-16 cb. yds (0.76-12 cubic meter)')
        self.assertEqual(diff['16.7-Cubic-yd'], '16.7-Cubic-yd (12.8 cubic meter)')
        self.assertEqual(diff['16,500-cu. yd'], '16,500-cu. yd (12,615 cubic meter)')
        self.assertEqual(res['body_html'], item['body_html'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .length_nautical_miles_to_metric import nautical_miles_to_metric


class MilesTestCase(unittest.TestCase):

    def test_m_to_km(self):
        text = 'Tom rides 125 nautical miles everyday. ' \
               'Tom rides 1 Nautical mile everyday. ' \
               'Tom rides 125 Nautical Miles everyday. ' \
               'Tom rides 125-nautical Miles everyday. ' \
               'Tom rides 12.5 nmi everyday. ' \
               'Tom rides 12.5nmi everyday. ' \
               'Tom rides 12.5-nmi everyday. ' \
               'Tom rides 12,500 nautical miles everyday. ' \
               'Tom rides 100-12500 Nautical Miles everyday. ' \

        item = {'body_html': text}
        res, diff = nautical_miles_to_metric(item)
        self.assertEqual(diff['125 nautical miles'], '125 nautical miles (231.5 km)')
        self.assertEqual(diff['1 Nautical mile'], '1 Nautical mile (1.9 km)')
        self.assertEqual(diff['125 Nautical Miles'], '125 Nautical Miles (231.5 km)')
        self.assertEqual(diff['125-nautical Miles'], '125-nautical Miles (231.5 km)')
        self.assertEqual(diff['12.5 nmi'], '12.5 nmi (23.2 km)')
        self.assertEqual(diff['12.5nmi'], '12.5nmi (23.2 km)')
        self.assertEqual(diff['12.5-nmi'], '12.5-nmi (23.2 km)')
        self.assertEqual(diff['12,500 nautical miles'], '12,500 nautical miles (23,150 km)')
        self.assertEqual(diff['100-12500 Nautical Miles'], '100-12500 Nautical Miles (185.2-23,150 km)')
        self.assertEqual(res['body_html'], item['body_html'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.utils import SuperdeskBaseEnum

DICTIONARY_FILE = 'file'


class DictionaryType(SuperdeskBaseEnum):
    """Defines Dictionary Type

    """

    DICTIONARY = 'dictionary'
    ABBREVIATIONS = 'abbreviations'


class DictionariesResource(Resource):
    """Dictionaries schema

    """

    schema = {
        'name': {
            'type': 'string',
            'required': True
        },
        'language_id': {
            'type': 'string',
            'required': True
        },
        'content': {
            'type': 'dict',
            'schema': {},
            'allow_unknown': True,
        },
        'content_list': {
            'type': 'string',
        },
        DICTIONARY_FILE: {
            'type': 'file',
        },
        'user': Resource.rel('users', nullable=True),
        'is_active': {
            'type': 'string',
            'default': 'true',
        },
        'type': {
            'type': 'string',
            'default': DictionaryType.DICTIONARY.value,
            'allowed': DictionaryType.values()
        },

        '_file_id': {'type': 'objectid', 'nullable': True, 'readonly': True},
        'init_version': {'type': 'integer'},
    }
    item_methods = ['GET', 'PATCH', 'PUT', 'DELETE']
    resource_methods = ['GET', 'POST', 'DELETE']
    privileges = {'POST': 'dictionaries', 'PATCH': 'dictionaries', 'DELETE': 'dictionaries'}
    etag_ignore_fields = ['content', 'content_list']



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.publish import SUBSCRIBER_TYPES

from superdesk.tests import TestCase
from superdesk.publish.transmitters.file_output import FilePublishService
from superdesk.errors import PublishFileError


class FilePublishServiceTest(TestCase):
    def setUp(self):
        self.fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)))
        self.subscribers = [{"_id": "1", "name": "Test", "media_type": "media",
                             "subscriber_type": SUBSCRIBER_TYPES.WIRE, "is_active": True,
                             "sequence_num_settings": {"max": 10, "min": 1},
                             "destinations": [{"name": "test", "delivery_type": "File", "format": "nitf",
                                               "config": {"file_path": self.fixtures}
                                               }]}]

    def test_file_write(self):
        item = {'item_id': 'test_file_name',
                'item_version': 1,
                'published_seq_num': 1,
                'formatted_item': 'I was here',
                'encoded_item': b'I was here',
                'item_encoding': 'utf-8',
                'destination': {"name": "test", "delivery_type": "File", "format": "nitf",
                                "config": {"file_path": self.fixtures, "file_extension": "txt"}}
                }
        service = FilePublishService()
        try:
            service._transmit(item, self.subscribers)
            self.assertTrue(True)
        finally:
            path = os.path.join(self.fixtures, 'test_file_name-1-1.txt')
            if os.path.isfile(path):
                os.remove(path)

    def test_format_default_file_extension(self):
        item = {'item_id': 'test_file_name',
                'item_version': 1,
                'published_seq_num': 1,
                'formatted_item': 'I was here',
                'encoded_item': b'I was here',
                'item_encoding': 'utf-8',
                'destination': {"name": "test", "delivery_type": "File", "format": "nitf",
                                "config": {"file_path": self.fixtures}}
                }
        service = FilePublishService()
        try:
            service._transmit(item, self.subscribers)
            self.assertTrue(True)
        finally:
            path = os.path.join(self.fixtures, 'test_file_name-1-1.ntf')
            if os.path.isfile(path):
                os.remove(path)

        item['destination']['config']['file_extension'] = ''
        try:
            service._transmit(item, self.subscribers)
            self.assertTrue(True)
        finally:
            path = os.path.join(self.fixtures, 'test_file_name-1-1.ntf')
            if os.path.isfile(path):
                os.remove(path)

    def test_file_write_fail(self):
        self.fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__) + '/xyz'))
        item = {'item_id': 'test_file_name',
                'item_version': 1,
                'formatted_item': 'I was here',
                'encoded_item': b'I was here',
                'item_encoding': 'utf-8',
                'destination': {"name": "test", "delivery_type": "File", "format": "nitf",
                                "config": {"file_path": self.fixtures}}
                }

        with self.app.app_context():
            service = FilePublishService()
            try:
                service._transmit(item, self.subscribers)
            except PublishFileError as ex:
                self.assertEqual(str(ex), 'PublishFileError Error 13000 - File publish error')
                self.assertEqual(ex.code, 13000)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
A module containing configuration of the Superdesk's production API.

The meaning of configuration options is described in the Eve framework
`documentation <http://python-eve.org/config.html#global-configuration>`_.
"""

from superdesk.default_settings import env, urlparse

from superdesk.default_settings import (  # noqa
    DEBUG,
    SUPERDESK_TESTING,
    MONGO_URI,
    ELASTICSEARCH_INDEX,
    ELASTICSEARCH_URL,
    AMAZON_ACCESS_KEY_ID,
    AMAZON_SECRET_ACCESS_KEY,
    AMAZON_REGION,
    AMAZON_CONTAINER_NAME,
    AMAZON_S3_SUBFOLDER,
    AMAZON_OBJECT_ACL,
    AMAZON_ENDPOINT_URL,
    strtobool
)

SECRET_KEY = env('PRODAPI_SECRET_KEY', '')

PRODAPI_INSTALLED_APPS = (
    'prod_api.items',
    'prod_api.assets',
    'prod_api.desks',
    'prod_api.planning',
    'prod_api.contacts',
    'prod_api.users',
)

# NOTE: no trailing slash for the PRODAPI_URL setting!
PRODAPI_URL = env('PRODAPI_URL', 'http://localhost:5500')
URL_PREFIX = env('PRODAPI_URL_PREFIX', 'prodapi')
API_VERSION = 'v1'
MEDIA_PREFIX = env('MEDIA_PREFIX', '{}/{}/{}/assets'.format(PRODAPI_URL.rstrip('/'), URL_PREFIX, API_VERSION))

# date formats
DATE_FORMAT = '%Y-%m-%dT%H:%M:%S+0000'
ELASTIC_DATE_FORMAT = '%Y-%m-%d'

# response in json
XML = False

# authorisation server
AUTH_SERVER_SHARED_SECRET = env('AUTH_SERVER_SHARED_SECRET', '')

# authentication
PRODAPI_AUTH_ENABLED = strtobool(env('PRODAPI_AUTH_ENABLED', 'true'))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .area_square_inch_to_metric import square_inch_to_metric


class AreaTestCase(unittest.TestCase):

    def test_f_to_c(self):
        text = 'Total area is 100.50 square inches for this land. '\
               'Total area is 15.7 square in for this land. '\
               'Total area is 1 Square Inch for this land. '\
               'Total area is 1-16 sq-in for this land. '\
               'Total area is 16.7-Square-in for this land. '\
               'Total area is 16,500-sq. in for this land. '\

        item = {'body_html': text}
        res, diff = square_inch_to_metric(item)
        self.assertEqual(diff['100.50 square inches'], '100.50 square inches (648.39 square cm)')
        self.assertEqual(diff['15.7 square in'], '15.7 square in (101.29 square cm)')
        self.assertEqual(diff['1 Square Inch'], '1 Square Inch (6.45 square cm)')
        self.assertEqual(diff['1-16 sq-in'], '1-16 sq-in (6.45-103.23 square cm)')
        self.assertEqual(diff['16.7-Square-in'], '16.7-Square-in (107.74 square cm)')
        self.assertEqual(diff['16,500-sq. in'], '16,500-sq. in (10.6 square meter)')
        self.assertEqual(res['body_html'], item['body_html'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
from . import unit_base
from decimal import Decimal
from flask_babel import lazy_gettext


def convert(feet, inches):
    """Converts from feet and inches to cm or m
    If feet contains '-' then inches won't have '-'
    If inches contains '-' then feet value will be 0
    :param feet: Feet value in string
    :param inches: Inch value in string
    :return: cm or m value in string, and the symbol as 'm' or 'cm'
    """
    foot_to_cm_rate = Decimal(30.48)
    inch_to_cm_rate = Decimal(2.54)
    total_centimeters = []
    symbol = 'cm'

    if '-' in feet:
        feet_list = feet.split('-')
        total_centimeters = [(Decimal(m) * foot_to_cm_rate) + (Decimal(inches) * inch_to_cm_rate) for m in feet_list]
    elif '-' in inches:
        inches_list = inches.split('-')
        total_centimeters = [(Decimal(i) * inch_to_cm_rate) for i in inches_list]
    else:
        # no multi values
        total_centimeters = [(Decimal(feet) * foot_to_cm_rate) + (Decimal(inches) * inch_to_cm_rate)]

    if any(c for c in total_centimeters if c > Decimal(100)):
        # if the value is greater than 100 then convert it to meter
        total_centimeters = [unit_base.format_converted((c / Decimal(100)), precision=2) for c in total_centimeters]
        symbol = 'm'
    else:
        total_centimeters = [unit_base.format_converted(c, precision=2) for c in total_centimeters]

    return '-'.join(total_centimeters), symbol


def do_conversion(item, converter, formatter, search_param):
    """Performs the conversion"""
    diff = {}

    # Group indexes
    match_index = 0  # Index of complete match i.e. 5' 10"
    value_index = 1  # Index of the value: contains feet if feet is in the match else inches if there's no feet
    feet_symbol_index = 7  # Index of feet symbol ', ft, feet, foot
    inches_with_feet_value_index = 11  # When there is a feet and inch value matched together
    inches_symbol_index = 5  # Index of inches symbol ", in, inch(es)

    def convert(match):
        match_item = match.group(match_index).strip()
        from_value = match.group(value_index)
        inches_from_value = '0'
        feet_symbol = match.group(feet_symbol_index)
        inches_symbol = match.group(inches_symbol_index)
        multi_values = '-' in from_value and from_value[-1:] != '-'

        if match_item and from_value:
            if feet_symbol:
                # check if any inches matched
                inches_from_value = match.group(inches_with_feet_value_index) or '0'
            elif inches_symbol:
                # no feet matching
                inches_from_value = from_value
                from_value = '0'
            else:
                return {}

            if not multi_values:
                from_value = re.sub(r'[^\d.]', '', from_value)
                inches_from_value = re.sub(r'[^\d.]', '', inches_from_value)
            to_value, symbol = converter(from_value, inches_from_value)
            diff.setdefault(match_item.strip(), formatter(match_item.strip(), to_value, symbol))
            return diff[match_item]

    for field in unit_base.macro_replacement_fields:
        if item.get(field, None):
            re.sub(search_param, convert, item[field])

    return (item, diff)


def feet_inches_to_metric(item, **kwargs):
    """Converts distance values from feet and inches to metric"""

    regex = r'(\d+-?,?\.?\d*)((\s*)|(-))(((\'|ft\.?|[fF]eet|[fF]oot)' \
        r'((-)|(\s*))(\d+)?\s?("|in)?)|(\"|[iI]nches|[iI]nch|in))'
    return do_conversion(item, convert, unit_base.format_output, regex)


name = 'feet_inches_to_metric'
label = lazy_gettext('Length feet-inches to metric')
callback = feet_inches_to_metric
access_type = 'frontend'
action_type = 'interactive'
group = lazy_gettext('length')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask_babel import lazy_gettext


def validate(item, **kwargs):
    """Checks if item has take_key value"""

    # validation
    if not item.get('anpa_take_key', '').strip():
        raise KeyError('Take key cannot be empty!')

    return item


name = 'take_key_validator'
label = lazy_gettext('validate take key')
callback = validate
access_type = 'backend'
action_type = 'direct'




import unittest
from decimal import Decimal

from superdesk.macros.currency_usd_to_cad import usd_to_cad


class CurrencyTestCase(unittest.TestCase):

    def test_usd_to_cad(self):
        item = {'body_html': '$100'}
        res, diff = usd_to_cad(item, rate=Decimal(2))
        self.assertTrue('$100' in diff)
        self.assertEqual(diff['$100'], '$100 (CAD 200)')



"""
Superdesk OAuth support

.. versionadded:: 1.9

There is ``oauth`` resource for authenticating users by email.
Email must be provided by trusted identity service, eg. Google.

The authentication flow with js client is:

- Client opens server api url in a new window,.
- There is a redirect to auth provider (eg. Google).
- Once authenticated user is redirected back to server api url.
- There user email from auth provider is used to authenticate user.
- Template is rendered with session data which sends message to parent window and popup is closed.

.. autofunction:: auth_user

"""

import logging
import superdesk

from flask import render_template, current_app as app

from apps.auth.auth import AuthResource
from apps.auth.service import AuthService
from superdesk.validation import ValidationError


RESOURCE = 'oauth'
TEMPLATE = 'oauth_authorized.html'


logger = logging.getLogger(__name__)


class OAuthResource(AuthResource):
    internal_resource = True


class OAuthService(AuthService):

    def authenticate(self, document):
        if not document.get('email'):
            return
        return superdesk.get_resource_service('auth_users').find_one(req=None, email=document['email'].lower())


def auth_user(email, userdata=None):
    """Authenticate user via email.

    This will create new session for user and render template with session data
    which is used by client to setup authentication.

    :param email: user email address
    """
    # we don't get email from service
    if userdata and userdata.get('email'):
        email = userdata['email']
    data = [{'email': email}]
    if not email:
        return render_template(TEMPLATE, data={'error': 404})
    try:
        superdesk.get_resource_service(RESOURCE).post(data)
        data[0]['_id'] = str(data[0]['_id'])
        data[0]['user'] = str(data[0]['user'])
        if userdata:
            superdesk.get_resource_service('users').update_external_user(data[0]['user'], userdata)
        return render_template(TEMPLATE, data=data[0])
    except ValueError:
        if not app.config['USER_EXTERNAL_CREATE'] or not userdata:
            return render_template(TEMPLATE, data={'error': 404})

    # create new user using userdata
    # and re-run auth
    try:
        user = superdesk.get_resource_service('users').create_external_user(userdata)
        return auth_user(user['email'])
    except ValidationError as err:  # can't create user, so let it fail on next iteration
        logger.error('can not create user automaticaly, userdata is not valid', extra={
            'userdata': userdata,
            'errors': err,
        })

    return auth_user(None)


def init_app(app):
    superdesk.register_resource(RESOURCE, OAuthResource, OAuthService, _app=app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import socket
import imaplib

from flask import current_app as app
from superdesk.errors import IngestEmailError
from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from superdesk.io.feeding_services import FeedingService
from superdesk.upload import url_for_media


class EmailFeedingService(FeedingService):
    """
    Feeding Service class which can read the article(s) from a configured mail box.
    """

    NAME = 'email'

    ERRORS = [IngestEmailError.emailError().get_error_description(),
              IngestEmailError.emailLoginError().get_error_description()]

    label = 'Email'

    fields = [
        {
            'id': 'server', 'type': 'text', 'label': 'Email Server',
            'placeholder': 'Email Server', 'required': True,
            'errors': {6003: 'Server not found.', 6002: 'Unexpected server response'}
        },
        {
            'id': 'port', 'type': 'text', 'label': 'Email Server Port',
            'placeholder': 'Email Server Port', 'required': True,
            'default': '993'
        },
        {
            'id': 'user', 'type': 'text', 'label': 'User',
            'placeholder': 'User', 'required': True
        },
        {
            'id': 'password', 'type': 'password', 'label': 'Password',
            'placeholder': 'Password', 'required': True,
            'errors': {6000: 'Authentication error.'}
        },
        {
            'id': 'mailbox', 'type': 'text', 'label': 'Mailbox',
            'placeholder': 'Mailbox', 'required': True,
            'errors': {6004: 'Authentication error.'}
        },
        {
            'id': 'formatted', 'type': 'boolean', 'label': 'Formatted Email Parser',
            'required': True
        },
        {
            'id': 'filter', 'type': 'text', 'label': 'Filter',
            'placeholder': 'Filter', 'required': True
        }
    ]

    def _test(self, provider):
        self._update(provider, update=None, test=True)

    def _update(self, provider, update, test=False):
        config = provider.get('config', {})
        server = config.get('server', '')
        port = int(config.get('port', 993))
        new_items = []

        try:
            try:
                socket.setdefaulttimeout(app.config.get('EMAIL_TIMEOUT', 10))
                imap = imaplib.IMAP4_SSL(host=server, port=port)
            except (socket.gaierror, OSError) as e:
                raise IngestEmailError.emailHostError(exception=e, provider=provider)

            try:
                imap.login(config.get('user', None), config.get('password', None))
            except imaplib.IMAP4.error:
                raise IngestEmailError.emailLoginError(imaplib.IMAP4.error, provider)

            try:
                rv, data = imap.select(config.get('mailbox', None), readonly=False)
                if rv != 'OK':
                    raise IngestEmailError.emailMailboxError()
                try:
                    rv, data = imap.search(None, config.get('filter', '(UNSEEN)'))
                    if rv != 'OK':
                        raise IngestEmailError.emailFilterError()
                    for num in data[0].split():
                        rv, data = imap.fetch(num, '(RFC822)')
                        if rv == 'OK' and not test:
                            try:
                                parser = self.get_feed_parser(provider, data)
                                new_items.append(parser.parse(data, provider))
                                rv, data = imap.store(num, '+FLAGS', '\\Seen')
                            except IngestEmailError:
                                continue
                finally:
                    imap.close()
            finally:
                imap.logout()
        except IngestEmailError:
            raise
        except Exception as ex:
            raise IngestEmailError.emailError(ex, provider)
        return new_items

    def prepare_href(self, href, mimetype=None):
        return url_for_media(href, mimetype)


register_feeding_service(EmailFeedingService)
register_feeding_service_parser(EmailFeedingService.NAME, 'email_rfc822')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import datetime, timedelta

from superdesk.tests import TestCase
from superdesk.io.commands.update_ingest import is_new_version


class UtilsTestCase(TestCase):

    def test_is_new_version(self):
        self.assertTrue(is_new_version({'version': 2}, {'version': 1}))
        self.assertTrue(is_new_version({'versiocreated': datetime.now()},
                                       {'versioncreated': datetime.now() - timedelta(days=1)}))
        self.assertTrue(is_new_version({'version': '10'}, {'version': '2'}))

        self.assertFalse(is_new_version({'version': 1}, {'version': 1}))
        self.assertFalse(is_new_version({'version': '123'}, {'version': '123'}))
        self.assertFalse(is_new_version({'versioncreated': datetime.now()},
                                        {'versioncreated': datetime.now()}))

    def test_is_new_version_content(self):
        self.assertTrue(is_new_version({'headline': 'foo'}, {}))
        self.assertTrue(is_new_version({'headline': 'foo'}, {'headline': 'bar'}))
        self.assertTrue(is_new_version(
            {'renditions': {'original': {'href': 'foo'}}},
            {'renditions': {'original': {'href': 'bar'}}},
        ))
        self.assertTrue(is_new_version(
            {'subject': [{'name': 'foo', 'qcode': 'foo'}]},
            {'subject': [{'name': 'bar', 'qcode': 'bar'}]},
        ))

        self.assertFalse(is_new_version({}, {}))
        self.assertFalse(is_new_version({'headline': 'foo'}, {'headline': 'foo', 'source': 'test'}))
        self.assertFalse(is_new_version(
            {'renditions': {'original': {'href': 'foo'}}},
            {'renditions': {'original': {'href': 'foo'}}},
        ))
        self.assertFalse(is_new_version(
            {'subject': [{'name': 'foo', 'qcode': 'foo'}]},
            {'subject': [{'name': 'foo', 'qcode': 'foo'}]},
        ))

    def test_is_new_version_ignores_expiry(self):
        yesterday = datetime.now() - timedelta(days=1)
        self.assertFalse(is_new_version(
            {'headline': 'foo', 'firstcreated': None, 'expiry': datetime.now()},
            {'headline': 'foo', 'firstcreated': yesterday, 'expiry': yesterday},
        ))




from unittest.mock import patch
from bson import ObjectId
from datetime import timedelta

from superdesk.utc import utcnow
from superdesk.tests import TestCase
from apps.auth.session_purge import RemoveExpiredSessions
from apps.auth import is_current_user_admin


class AuthTestCase(TestCase):

    def test_remove_expired_sessions_syncs_online_users(self):
        sess_id = ObjectId()
        user_ids = self.app.data.insert('users', [
            {'name': 'foo', 'username': 'foo', 'session_preferences': {'test': {}}},
            {'name': 'bar', 'username': 'bar', 'session_preferences': {'bar': {}, str(sess_id): {}}},
        ])

        self.assertEqual(2, len(user_ids))

        self.app.data.insert('auth', [
            {'user': user_ids[0], '_updated': utcnow() - timedelta(days=50)},
            {'user': user_ids[1], '_updated': utcnow()},
            {'_id': sess_id, 'user': user_ids[1], '_updated': utcnow()}
        ])

        RemoveExpiredSessions().run()

        users = self.app.data.find_list_of_ids('users', user_ids)

        self.assertEqual({}, users[0]['session_preferences'])
        self.assertEqual({str(sess_id): {}}, users[1]['session_preferences'])

    def test_is_current_user_admin(self):
        with patch('apps.auth.get_user', return_value={}):
            self.assertFalse(is_current_user_admin())

        with patch('apps.auth.get_user', return_value={'user_type': 'administrator'}):
            self.assertTrue(is_current_user_admin())

    def test_session_expiry_date_update(self):
        user_ids = self.app.data.insert('users', [
            {'username': 'foo', 'user_type': 'administrator'},
        ])
        self.app.data.insert('auth', [
            {'user': user_ids[0], '_updated': utcnow() - timedelta(minutes=5), 'token': 'foo'},
        ])

        with self.app.test_request_context('/users', method='POST'):
            self.app.auth.check_auth('foo', [], 'users', 'POST')
            auth = self.app.data.find_one('auth', None, token='foo')
            self.assertGreaterEqual(auth['_updated'], utcnow() - timedelta(seconds=1))

            self.app.data.update('auth', auth['_id'], {'_updated': utcnow() - timedelta(seconds=10)}, auth)

            self.app.auth.check_auth('foo', [], 'users', 'POST')
            auth = self.app.data.find_one('auth', None, token='foo')
            self.assertLess(auth['_updated'], utcnow() - timedelta(seconds=1))



import pytest
import json
from flask import url_for

from superdesk import get_resource_service


def test_service_get(prodapi_app_with_data):
    """
    Test fetching items using `events_history` service
    :param prodapi_app_with_data: prod api app with filled data
    """

    with prodapi_app_with_data.app_context():
        items_service = get_resource_service('events_history')

        assert len(list(items_service.get(req=None, lookup={}))) == 2


def test_readonly(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that `events_history` endpoint is readonly
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        for method, status in (('get', 200), ('post', 405), ('patch', 405), ('put', 405), ('delete', 405)):
            # we send a request
            resp = getattr(prodapi_app_with_data_client, method)(
                url_for('events_history|resource')
            )
            # we get a response
            assert resp.status_code == status


def test_excluded_fields(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that fields which are listed as `excluded_fields` are not in the response.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        # list
        resp = prodapi_app_with_data_client.get(url_for('events_history|resource'))
        resp_data = json.loads(resp.data.decode('utf-8'))
        item = resp_data['_items'][0]

        with pytest.raises(KeyError):
            item['update']['_etag']

        with pytest.raises(KeyError):
            item['update']['_links']

        with pytest.raises(KeyError):
            item['update']['_status']

        with pytest.raises(KeyError):
            item['update']['_updated']

        with pytest.raises(KeyError):
            item['update']['_created']

        with pytest.raises(KeyError):
            item['update']['_id']




import flask
import unittest
import superdesk.tests as tests
import superdesk.auth.saml as saml

from unittest.mock import patch


SAML_DATA = {
    "http://schemas.microsoft.com/identity/claims/tenantid": ["3cd37c1d"],
    "http://schemas.microsoft.com/identity/claims/objectidentifier": ["2f2a99f7"],
    "http://schemas.microsoft.com/identity/claims/displayname": ["Bar, Foo"],
    "http://schemas.microsoft.com/identity/claims/identityprovider": ["https://sts.windows.net/3cd37c1d/"],
    "http://schemas.microsoft.com/claims/authnmethodsreferences": [
        "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport",
    ],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname": ["Foo"],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname": ["Bar"],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress": ["foo.bar@example.com"],
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": ["foo@bar.com"],
    "http://schemas.microsoft.com/ws/2008/06/identity/claims/role": ["editor"],
    "displayname": ["Bar, Foo"],
    "country": ["CA"],
    "city": ["Toronto"],
    "department": ["IT"]
}


ERROR = '{"error": 404}'


class SamlAuthTestCase(tests.TestCase):

    def setUp(self):
        self.desks = [{'name': 'Sports'}, {'name': 'Finance'}]
        self.roles = [{'name': 'Editor'}, {'name': 'Admin'}]
        with self.app.app_context():
            self.app.data.insert('desks', self.desks)
            self.app.data.insert('roles', self.roles)

    @patch('superdesk.auth.saml.init_saml_auth')
    def test_create_missing_user(self, init_mock):
        with self.app.test_client() as c:
            flask.session[saml.SESSION_NAME_ID] = 'foo.bar@example.com'
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA

            resp = saml.index()
            self.assertIn(ERROR, resp)

            with patch.dict(self.app.config, {
                'USER_EXTERNAL_CREATE': True,
                'USER_EXTERNAL_DESK': 'sports',
                'USER_EXTERNAL_USERNAME_STRIP_DOMAIN': True,
            }):
                resp = saml.index()
            self.assertNotIn(ERROR, resp)

            user = self.app.data.find_one('users', req=None, email='foo.bar@example.com')
            self.assertTrue(user.get('is_active'))
            self.assertTrue(user.get('is_enabled'))
            self.assertFalse(user.get('needs_activation'))
            self.assertEqual('user', user.get('user_type'))
            self.assertEqual('foo', user['username'])

            self.assertIsNotNone(user.get('role'))
            role = self.app.data.find_one('roles', req=None, _id=user['role'])
            self.assertEqual('Editor', role['name'])

            self.assertIsNotNone(user.get('desk'))
            desk = self.app.data.find_one('desks', req=None, _id=user['desk'])
            self.assertIn({'user': user['_id']}, desk.get('members'))

    @patch('superdesk.auth.saml.init_saml_auth')
    def test_create_missing_user_missing_userdata(self, init_mock):
        with self.app.test_client() as c:
            # with missing data it can't work
            flask.session[saml.SESSION_NAME_ID] = 'foo.bar@example.com'
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA.copy()
            flask.session[saml.SESSION_USERDATA_KEY].update({
                "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": [],
            })
            with patch.dict(self.app.config, {'USER_EXTERNAL_CREATE': True}):
                resp = saml.index()
            self.assertIn(ERROR, resp)

    @patch('superdesk.auth.saml.init_saml_auth')
    def test_handle_saml_name_id_not_email(self, init_mock):
        with self.app.test_client() as c:
            # with missing data it can't work
            flask.session[saml.SESSION_NAME_ID] = 'something_weird_like_guid'
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA.copy()
            with patch.dict(self.app.config, {'USER_EXTERNAL_CREATE': True}):
                resp = saml.index()
            self.assertNotIn(ERROR, resp)

    @patch('superdesk.auth.saml.init_saml_auth')
    def test_update_user_data_when_it_changes(self, init_mock):
        with self.app.test_client() as c:
            # with missing data it can't work
            flask.session[saml.SESSION_NAME_ID] = 'nameId'
            flask.session[saml.SESSION_USERDATA_KEY] = SAML_DATA.copy()
            with patch.dict(self.app.config, {
                'USER_EXTERNAL_CREATE': True,
                'USER_EXTERNAL_DESK': 'sports',
            }):
                resp = saml.index()

            user = self.app.data.find_one('users', req=None, email='foo.bar@example.com')
            self.assertIsNotNone(user)
            self.assertEqual(self.desks[0]['_id'], user['desk'])

            self.app.data.update('users', user['_id'], {
                'user_type': 'administrator',
                'desk': self.desks[1]['_id'],
            }, user)

            flask.session[saml.SESSION_USERDATA_KEY].update({
                "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname": ["John"],
                "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname": ["Doe"],
                "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress": ["foo.bar@example.com"],
                "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": ["john"],
                "displayname": ["Doe, John"],
            })

            with patch.dict(self.app.config, {
                'USER_EXTERNAL_CREATE': True,
                'USER_EXTERNAL_DESK': 'sports',
            }):
                resp = saml.index()

        user = self.app.data.find_one('users', req=None, email='foo.bar@example.com')
        self.assertEqual('John', user['first_name'])
        self.assertEqual('Doe', user['last_name'])
        self.assertEqual('Doe, John', user['display_name'])
        self.assertEqual('administrator', user['user_type'])
        self.assertEqual(self.desks[1]['_id'], user['desk'])




from superdesk.resource import Resource

TODAY_DATE = 'now/d'
WEEK_DATE = 'now/w'

allowed_times = ['now-{0}h'.format(hour) for hour in range(1, 25)]
allowed_times.append(TODAY_DATE)
allowed_times.append(WEEK_DATE)


class HighlightsResource(Resource):
    """Highlights schema"""

    schema = {
        'name': {
            'type': 'string',
            'iunique': True,
            'required': True
        },
        'template': Resource.rel('content_templates', nullable=True),
        'desks': {
            'type': 'list',
            'schema': Resource.rel('desks', True)
        },
        'auto_insert': {
            'type': 'string',
            'allowed': allowed_times,
            'default': TODAY_DATE,
        },
        'groups': {
            'type': 'list',
            'schema': {
                'type': 'string'
            }
        }
    }
    privileges = {'POST': 'highlights', 'PATCH': 'highlights', 'DELETE': 'highlights'}

    mongo_indexes = {
        'name_1': ([('name', 1)], {'unique': True}),
    }


class MarkedForHighlightsResource(Resource):
    """Marked for highlights Schema"""

    schema = {
        'highlights': {
            'type': 'string',
            'required': True
        },
        'marked_item': {
            'type': 'string',
            'required': True
        }
    }
    privileges = {'POST': 'mark_for_highlights'}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import json
from unittest.mock import patch

from apps.prepopulate.app_populate import AppPopulateCommand
from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.vocabularies import VocabulariesService
from superdesk.errors import SuperdeskApiError


class VocabulariesPopulateTest(TestCase):

    def setUp(self):
        self.filename = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'vocabularies.json')
        self.json_data = [
            {'_id': 'categories',
             'unique_field': 'qcode',
             'items': [
                 {'name': 'National', 'qcode': 'A', 'is_active': True},
                 {'name': 'Domestic Sports', 'qcode': 'T', 'is_active': False}
             ]},
            {'_id': 'newsvalue',
             'items': [
                 {'name': '1', 'value': '1', 'is_active': True},
                 {'name': '2', 'value': '2', 'is_active': True},
                 {'name': '3', 'value': '3', 'is_active': False}
             ]}
        ]

        with open(self.filename, 'w+') as file:
            json.dump(self.json_data, file)

    def test_populate_vocabularies(self):
        cmd = AppPopulateCommand()
        cmd.run(self.filename)
        service = get_resource_service('vocabularies')

        for item in self.json_data:
            data = service.find_one(_id=item['_id'], req=None)
            self.assertEqual(data['_id'], item['_id'])
            self.assertListEqual(data['items'], item['items'])

    def test_check_uniqueness(self):
        items = [{'name': 'National', 'qcode': 'A', 'is_active': True},
                 {'name': 'Domestic Sports', 'qcode': 'a', 'is_active': True}]
        with self.assertRaises(SuperdeskApiError):
            VocabulariesService()._check_uniqueness(items, 'qcode')

    def test_check_uniqueness_active_only(self):
        items = [{'name': 'National', 'qcode': 'A', 'is_active': True},
                 {'name': 'Domestic Sports', 'qcode': 'A', 'is_active': False}]
        VocabulariesService()._check_uniqueness(items, 'qcode')

    def test_check_value_of_unique_field(self):
        items = [{'name': 'National', 'is_active': True},
                 {'name': 'Domestic Sports', 'qcode': 'A', 'is_active': True}]
        with self.assertRaises(SuperdeskApiError):
            VocabulariesService()._check_uniqueness(items, 'qcode')

    def test_get_rightsinfo(self):
        service = get_resource_service('vocabularies')
        vocab = {
            '_id': 'rightsinfo',
            'items': [
                {
                    'is_active': True,
                    'name': 'default',
                    'copyrightHolder': 'default holder',
                    'copyrightNotice': 'default notice',
                    'usageTerms': 'default terms'
                },
                {
                    'is_active': True,
                    'name': 'foo',
                    'copyrightHolder': 'foo holder',
                    'copyrightNotice': 'foo notice',
                    'usageTerms': 'foo terms'
                },
            ]
        }

        with patch.object(service, 'find_one', return_value=vocab):
            info = service.get_rightsinfo({})
            self.assertEqual('default holder', info['copyrightholder'])
            self.assertEqual('default notice', info['copyrightnotice'])
            self.assertEqual('default terms', info['usageterms'])
            info = service.get_rightsinfo({'source': 'foo'})
            self.assertEqual('foo holder', info['copyrightholder'])
            self.assertEqual('foo notice', info['copyrightnotice'])
            self.assertEqual('foo terms', info['usageterms'])

    def test_get_locale_vocabulary(self):
        items = [
            {'is_active': True, 'name': 'FIXME1', 'qcode': 'f', 'subject': '',
             'translations': {'name': {'fr': 'FIXME1-fr', 'es': 'FIXME1-es'}}},
            {'is_active': True, 'name': 'FIXME2', 'qcode': 'f', 'subject': '',
             'translations': {'name': {'fr': 'FIXME2-fr', 'es': 'FIXME2-es'}}}
        ]
        result = VocabulariesService().get_locale_vocabulary(items, 'fr')

        self.assertEqual(result[0]['name'], 'FIXME1-fr')
        self.assertEqual(result[1]['name'], 'FIXME2-fr')

    def tearDown(self):
        os.remove(self.filename)



import io
import logging
import mimetypes

import magic
from werkzeug import datastructures


logger = logging.getLogger(__name__)


class MimetypeMixin:

    def _get_mimetype(self, content, filename=None, content_type=None):
        """
        Return mimetype of the `content` and as a fallback using `filename`

        :param content: binary stream
        :type stream: `io.BytesIO` | `io.BufferedReader` | `io.BufferedIOBase` | `werkzeug.datastructures.FileStorage`
        :param filename: filename
        :type filename: str
        :param content_type: expected content type, used as a fallback
        :type filename: str
        """

        if content_type:
            for media_type in ('image', 'video', 'audio'):
                if content_type.startswith(media_type):
                    return content_type

        determined_content_type = None

        try:
            stream = content
            if type(content) is datastructures.FileStorage:
                stream = content.stream
            stream_type = type(stream)
            try:
                # we expect types with `io.BufferedIOBase` interface
                bytes_buffer = stream.read()
                stream.seek(0)
            except AttributeError:
                msg = 'Not expected format for incoming binary stream: {}'.format(stream_type)
                logger.warning(msg)
                raise Exception(msg)
            # detect mimetype using wrapper around libmagic
            determined_content_type = magic.Magic(mime=True).from_buffer(bytes_buffer)

            # if 'application/octet-stream' is returned it means that libmagic was not able to
            # detect mimetype precisely and as a fallback 'application/octet-stream' was returned.
            # in this case we should try to detect a mimetype by filename
            if determined_content_type == 'application/octet-stream':
                msg = 'libmagic was not able to detect mimetype precisely'
                raise Exception(msg)
        except Exception as e:
            logger.warning(e)
            if filename:
                # detect mimetype using filename extension
                determined_content_type = mimetypes.MimeTypes().guess_type(filename)[0]

        if determined_content_type and determined_content_type != content_type:
            logger.info("Content type '{}' was expected, but '{}' was determined")
            content_type = determined_content_type

        return content_type



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from . import unit_base
from decimal import Decimal
from flask_babel import lazy_gettext

KILOMETER_SYMBOL = 'km'


def convert(miles, **kwargs):
    """
    Converts from miles to kilometers
    :param miles: Miles value in string
    :return: Kilometers value in string
    """
    miles_to_km_rate = Decimal(1.609344)
    miles_list = miles.split('-')
    kilometers = [unit_base.format_converted((Decimal(m) * miles_to_km_rate), precision=1) for m in miles_list]
    return '-'.join(kilometers), KILOMETER_SYMBOL


def miles_to_metric(item, **kwargs):
    """Converts distance values from miles to metric"""

    regex = r'(\d+-?,?\.?\d*)((\s*)|(-))((mi)|([mM]iles?))\b'
    return unit_base.do_conversion(item, convert, unit_base.format_output, regex, match_index=0, value_index=1)


name = 'miles_to_metric'
label = lazy_gettext('Length miles to kilometres')
order = 4
callback = miles_to_metric
access_type = 'frontend'
action_type = 'interactive'
group = lazy_gettext('length')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from unittest.mock import Mock

from superdesk.publish.transmitters.email import EmailPublishService
from superdesk.tests import TestCase
from superdesk.publish import init_app


class MockMediaFS:
    """Simple Mocked Media FS"""

    def __init__(self):
        self.get = Mock()
        self.get.side_effect = self._get
        self.read = Mock()
        self.read.side_effect = self._read

    name = 'filename'
    content_type = 'image/jpeg'

    def _get(self, id, resource):
        return self

    def _read(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', 'IPTC-PhotometadataRef-Std2017.1.jpg'))
        with open(fixture, 'rb') as f:
            return f.read()


class MockMail:

    def __init__(self):
        self.send = Mock()
        self.send.side_effect = self._send

    def _send(self, message):
        if message.subject != 'Test Subject':
            raise ValueError('Unexpected Subjet')
        if len(message.attachments) != 1:
            raise Exception('Wrong number of attachments')


class EmailPublishServiceTest(TestCase):
    filename = "IPTC-PhotometadataRef-Std2017.1.jpg"

    def setUp(self):
        init_app(self.app)
        self._media = self.app.media
        self.app.media = MockMediaFS()
        self._mail = self.app.mail
        self.app.mail = MockMail()

    def tearDown(self):
        self.app.mail = self._mail
        self.app.media = self._media

    def test_attachment(self):
        queue_item = {
            'item_id': '123',
            'destination':
            {
                'delivery_type': 'email',
                'config': {
                    'media_cid': 'MainImage',
                    'media_rendition': 'viewImage',
                    'recipients': 'a@b.c.d',
                    'attach_media': True,
                    'watermark': True
                },
                'name': 'Email',
                'format': 'Email'
            },
            'formatted_item': '{"message_text": "Test", "message_html": "<p>Test</p>", '
                              '"message_subject": "Test Subject", "renditions": {"viewImage": {"media": "1234"}}}'
        }

        transmitter = EmailPublishService()
        transmitter._transmit(queue_item=queue_item, subscriber={})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from eve.utils import config
from superdesk import get_resource_service
from planning.common import ASSIGNMENT_WORKFLOW_STATE
from copy import deepcopy
from flask_babel import lazy_gettext


def update_on_assign_id(item, **kwargs):
    assign_id = item.get('assignment_id')
    if assign_id:
        assignments_service = get_resource_service('assignments')
        assignment = assignments_service.find_one(req=None, _id=assign_id)
        if assignment is None:
            return item
        if assignment['assigned_to']['state'] == ASSIGNMENT_WORKFLOW_STATE.ASSIGNED:
            updates = {'assigned_to': deepcopy(assignment.get('assigned_to'))}
            updates['assigned_to']['state'] = ASSIGNMENT_WORKFLOW_STATE.IN_PROGRESS
            assignments_service.patch(assignment[config.ID_FIELD], updates)
    return item


name = 'assign_status'
label = lazy_gettext('Update Status On Assignment ID')
callback = update_on_assign_id
access_type = 'backend'
action_type = 'direct'



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
import json
import bson
from .delete_archived_document import DeleteArchivedDocumentCommand
import ast


class UpdateArchivedDocumentCommand(superdesk.Command):
    """
    Update metadata for a document in both Mongodb and ElasticSearch by supplying ids
    field to update and the value for the update

    --field   Field name
    --value   Value to be set in the field
    --parseNeeded   Optional. True if value is a complex type, not an int or string

    Example:
    ::

        $ manage.py app:updateArchivedDocument --ids='["588c1df11d41c80928015601","588c1b901d41c805dce70df0"]'
            --field=anpa_category
            --value=[{"scheme" : null,"qcode" : "f","subject" : "04000000","name" : "Finance"}]
            --parseNeeded=True

    """

    option_list = [
        superdesk.Option('--ids', '-i', dest='ids', required=True),
        superdesk.Option('--field', '-f', dest='field', required=True),
        superdesk.Option('--value', '-v', dest='value', required=True),
        superdesk.Option('--parseNeeded', '-p', dest='parseNeeded', default=False)
    ]

    def run(self, ids, field, value, parseNeeded=False):
        ids = ast.literal_eval(ids)

        if parseNeeded:
            try:
                value = json.loads(value)
            except Exception as e:
                print('Error in parsing the value: {}'.format(value))
                print(e)
                return

        if ids and len(ids) > 0:
            items = DeleteArchivedDocumentCommand().get_archived_items(ids)

            for item in items:
                superdesk.get_resource_service('archived').system_update(bson.ObjectId(item['_id']),
                                                                         {field: value},
                                                                         item)
                print('Archived item {} has been updated.'.format(item['_id']))
                print('-' * 45)


superdesk.command('app:updateArchivedDocument', UpdateArchivedDocumentCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock

from superdesk.tests import TestCase
from content_api.publish.service import PublishService
from apps.publish.enqueue.enqueue_service import EnqueueService
from apps.packages.package_service import PackageService
from apps.archive.archive import ArchiveService


def _fake_extend_subscriber_items(self,
                                  subscriber_items,
                                  subscribers,
                                  package_item,
                                  package_item_id,
                                  subscriber_codes):
    subscriber_items.clear()
    subscriber_items.update({'8': {'subscriber': {
        '_id': 'toto',
        'is_targetable': True,
        'products': [],
        'api_products': ['570e04e23c5e9f89fe95366e'],
        'name': 'content_api',
        'subscriber_type': 'all',
        'destinations': [],
        'is_active': True,
        'sequence_num_settings': {'min': 1, 'max': 999},
        'api_enabled': True},
        'items': {'8': '8'},
        'codes': []}})


class EnqueueServiceTest(TestCase):
    queue_items = [
        {
            "_id": 1,
            "destination": {
                "delivery_type": "ftp",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub1",
            "state": "pending",
            "associated_items": ["123"],
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '1',
            "item_version": 1,
            "publishing_action": "published"
        },
        {
            "_id": 2,
            "destination": {
                "delivery_type": "ftp",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub1",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '1',
            "item_version": 1,
            "publishing_action": "corrected",
            "associated_items": ["123"]
        },
        {
            "_id": 3,
            "destination": {
                "delivery_type": "ftp",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub2",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '2',
            "item_version": 1,
            "publishing_action": "published",
            "associated_items": ["123"]
        },
        {
            "_id": 4,
            "destination": {
                "delivery_type": "content_api",
                "format": "ninjs",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub2",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '2',
            "item_version": 2,
            "publishing_action": "corrected",
            "associated_items": ["456"]
        },
        {
            "_id": 5,
            "destination": {
                "delivery_type": "ftp",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub3",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '3',
            "item_version": 1,
            "publishing_action": "published",
            "associated_items": ["786"]
        },
        {
            "_id": 6,
            "destination": {
                "delivery_type": "content_api",
                "format": "ninjs",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub3",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '3',
            "item_version": 2,
            "publishing_action": "corrected"
        },
        {
            "_id": 7,
            "destination": {
                "delivery_type": "ftp",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub4",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '2',
            "item_version": 1,
            "publishing_action": "published",
            "associated_items": ["123"]
        },
        {
            "_id": 8,
            "destination": {
                "delivery_type": "content_api",
                "format": "ninjs",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub4",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '2',
            "item_version": 2,
            "publishing_action": "corrected",
            "associated_items": ["456"]
        },
        {
            "_id": 9,
            "destination": {
                "delivery_type": "content_api",
                "format": "ninjs",
                "config": {},
                "name": "destination1"
            },
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub5",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": '5',
            "item_version": 1,
            "publishing_action": "published"
        }
    ]
    content_api_package = {
        "_id": "10",
        "destination": {
            "delivery_type": "content_api",
            "format": "ninjs",
            "config": {},
            "name": "destination1"
        },
        "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
        "version": "2",
        "type": "composite",
        "headline": "test headline",
        "urgency": 3,
        "pubstatus": "usable",
        "slugline": "test slugline",
        "source": "foo",
        "associations": {"main-0": {"guid": "toto", "type": "text"},
                         "main-1": {"guid": "titi", "type": "text"}},
        "priority": 6,
        "subject": [{"code": "03003000", "name": "famine"}],
        "service": [{"code": "f", "name": "FIXME"}],
        "copyrightholder": "",
        "copyrightnotice": "",
        "usageterms": "",
        "state": "published",
        "genre": [{"name": "Article (news)", "code": "Article"}]}

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert('publish_queue', self.queue_items)

    def test_previously_sent_item_association_for_one_subscriber(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = \
            service._get_subscribers_for_previously_sent_items({'item_id': '1'})
        self.assertEqual(len(associated_items.keys()), 1)
        self.assertIn('sub1', list(associated_items.keys()))
        self.assertIn('123', associated_items['sub1'])

    def test_previously_sent_item_association_for_multiple_subscribers(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = \
            service._get_subscribers_for_previously_sent_items({'item_id': '2'})
        self.assertEqual(len(associated_items.keys()), 2)
        self.assertIn('sub2', associated_items)
        self.assertIn('sub4', associated_items)
        self.assertIn('123', associated_items['sub2'])
        self.assertIn('456', associated_items['sub2'])
        self.assertIn('123', associated_items['sub4'])
        self.assertIn('456', associated_items['sub4'])

    def test_previously_sent_item_association_for_removed_associations(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = \
            service._get_subscribers_for_previously_sent_items({'item_id': '3'})
        self.assertEqual(len(associated_items.keys()), 1)
        self.assertIn('sub3', list(associated_items.keys()))
        self.assertIn('786', associated_items['sub3'])

    def test_previously_sent_item_association_for_no_associations(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = \
            service._get_subscribers_for_previously_sent_items({'item_id': '5'})
        self.assertEqual(len(associated_items.keys()), 0)

    @mock.patch.object(PackageService, 'get_residrefs', lambda self, package: ['8', '9'])
    @mock.patch.object(ArchiveService, 'find_one', lambda self, req, **lookup: EnqueueServiceTest.content_api_package)
    @mock.patch.object(EnqueueService, '_extend_subscriber_items', _fake_extend_subscriber_items)
    @mock.patch.object(EnqueueService, 'queue_transmission', lambda *a, **kw: ([], True))
    @mock.patch.object(PublishService, 'publish')
    def test_content_api_package_publishing(self, content_api_publish):
        service = EnqueueService()
        service.enqueue_item(self.content_api_package)
        # Mock.assert_called_once is only available in Python 3.6
        # so we emulate it by counting the number of calls
        assert content_api_publish.call_count == 1



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from flask import abort, current_app as app
from eve.utils import config
from apps.archive.common import copy_metadata_from_user_preferences
from superdesk.media.media_operations import process_file_from_stream, decode_metadata
from superdesk.media.renditions import generate_renditions, delete_file_on_error, get_renditions_spec
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE, ITEM_TYPE, CONTENT_TYPE
from superdesk.upload import url_for_media
from .common import update_dates_for, generate_guid, GUID_TAG, set_original_creator, \
    generate_unique_id_and_name, set_item_expiry
from superdesk.activity import add_activity
from superdesk.filemeta import set_filemeta
from superdesk.timer import timer
from superdesk.media.video_editor import VideoEditorWrapper

logger = logging.getLogger(__name__)


class ArchiveMediaService():
    video_editor = VideoEditorWrapper()

    type_av = {'image': CONTENT_TYPE.PICTURE, 'audio': CONTENT_TYPE.AUDIO, 'video': CONTENT_TYPE.VIDEO}

    def on_create(self, docs):
        """Create corresponding item on file upload."""

        for doc in docs:
            if 'media' not in doc or doc['media'] is None:
                abort(400, description="No media found")
            # check content type of video by python-magic
            content_type = app.media._get_mimetype(doc['media'])
            doc['media'].seek(0)
            file_type = content_type.split('/')[0]
            if file_type == 'video' and app.config.get("VIDEO_SERVER_ENABLED"):
                # upload media to video server
                res, renditions, metadata = self.upload_file_to_video_server(doc)
                # get thumbnails for timeline bar
                self.video_editor.create_timeline_thumbnails(doc.get('media'), 60)
            else:
                file, content_type, metadata = self.get_file_from_document(doc)
                inserted = [doc['media']]
                # if no_custom_crops is set to False the custom crops are generated automatically on media upload
                # see (SDESK-4742)
                rendition_spec = get_renditions_spec(no_custom_crops=app.config.get("NO_CUSTOM_CROPS"))
                with timer('archive:renditions'):
                    renditions = generate_renditions(file, doc['media'], inserted, file_type,
                                                     content_type, rendition_spec, url_for_media)
            try:
                self._set_metadata(doc)
                doc[ITEM_TYPE] = self.type_av.get(file_type)
                doc[ITEM_STATE] = CONTENT_STATE.PROGRESS
                doc['renditions'] = renditions
                doc['mimetype'] = content_type
                set_filemeta(doc, metadata)
                add_activity('upload', 'uploaded media {{ name }}',
                             'archive', item=doc,
                             name=doc.get('headline', doc.get('mimetype')),
                             renditions=doc.get('renditions'))
            except Exception as io:
                logger.exception(io)
                for file_id in inserted:
                    delete_file_on_error(doc, file_id)
                if res:
                    self.video_editor.delete(res.get('_id'))
                abort(500)

    def _set_metadata(self, doc):
        """
        Adds metadata to doc.
        """

        update_dates_for(doc)
        generate_unique_id_and_name(doc)
        doc['guid'] = generate_guid(type=GUID_TAG)
        doc.setdefault(config.ID_FIELD, doc['guid'])
        doc[config.VERSION] = 1
        set_item_expiry({}, doc)

        if not doc.get('_import', None):
            set_original_creator(doc)

        doc.setdefault(ITEM_STATE, CONTENT_STATE.DRAFT)

        if not doc.get('ingest_provider'):
            doc['source'] = app.config.get('DEFAULT_SOURCE_VALUE_FOR_MANUAL_ARTICLES')

        copy_metadata_from_user_preferences(doc)

    def get_file_from_document(self, doc):
        file = doc.get('media_fetched')
        if file:
            del doc['media_fetched']
        else:
            content = doc['media']
            res = process_file_from_stream(content, content_type=content.mimetype)
            file_name, content_type, metadata = res
            logger.debug('Going to save media file with %s ' % file_name)
            content.seek(0)
            with timer('media:put.original'):
                doc['media'] = app.media.put(content, filename=file_name, content_type=content_type, metadata=metadata)
            return content, content_type, decode_metadata(metadata)

        return file, file.content_type, file.metadata

    def upload_file_to_video_server(self, doc):
        """
        Upload file to video server and create rendition for it
        :param doc: info of file
        :return:
        """
        # upload video to video server
        res = self.video_editor.create(doc.get('media'))
        doc['media'] = res['_id']
        metadata = res.get('metadata')
        # create renditions
        rend = {
            'href': res.get('url'),
            'video_editor_id': res.get('_id'),
            'mimetype': res.get('mime_type'),
            'version': res.get('version'),
        }
        renditions = {'original': rend}
        return res, renditions, metadata



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import unittest
import elasticsearch
from content_api.app import get_app
from content_api.app.settings import ELASTICSEARCH_URL
from eve_elastic import get_es, get_indices


TEST_DBNAME = 'content-api-tests'


def get_test_settings():
    test_settings = {}
    test_settings['ELASTICSEARCH_URL'] = ELASTICSEARCH_URL
    test_settings['ELASTICSEARCH_INDEX'] = TEST_DBNAME
    test_settings['MONGO_DBNAME'] = TEST_DBNAME
    test_settings['MONGO_URI'] = 'mongodb://localhost/%s' % TEST_DBNAME
    test_settings['DEBUG'] = True
    test_settings['TESTING'] = True
    test_settings['SUPERDESK_TESTING'] = True
    test_settings['CONTENT_EXPIRY_MINUTES'] = 99
    test_settings['SUPERDESK_CONTENTAPI_TESTING'] = True
    test_settings['REDIS_URL'] = 'redis://localhost:6379'
    test_settings['BCRYPT_GENSALT_WORK_FACTOR'] = 12

    return test_settings


def drop_elastic(app):
    with app.app_context():
        try:
            es = get_es(app.config['ELASTICSEARCH_URL'])
            get_indices(es).delete(app.config['ELASTICSEARCH_INDEX'])
        except elasticsearch.exceptions.NotFoundError:
            pass


def drop_mongo(app):
    with app.test_request_context():
        try:
            app.data.mongo.pymongo().cx.drop_database(app.config['MONGO_DBNAME'])
        except AttributeError:
            pass


def setup(context=None, config=None):
    app_config = get_test_settings()
    if config:
        app_config.update(config)

    app = get_app(app_config)
    drop_elastic(app)
    drop_mongo(app)

    # create index again after dropping it
    app.data.elastic.init_app(app)
    app.data.elastic.init_index()

    if context:
        context.app = app
        context.client = app.test_client()


def set_placeholder(context, name, value):
    old_p = getattr(context, 'placeholders', None)
    if not old_p:
        context.placeholders = dict()
    context.placeholders[name] = value


def get_prefixed_url(current_app, endpoint):
    if endpoint.startswith('http://'):
        return endpoint

    endpoint = endpoint if endpoint.startswith('/') else ('/' + endpoint)
    url = current_app.config['URL_PREFIX'] + endpoint
    return url


def get_fixture_path(filename):
    rootpath = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    return os.path.join(rootpath, 'features', 'steps', 'fixtures', filename)


class TestCase(unittest.TestCase):
    def setUp(self):
        setup(self)



from lxml import etree

from .story import Story


class StoryTable(Story):
    """
    Story which represents `table` html tag.
    """

    TABLE_DEFAULTS = {
        'HeaderRowCount': '0',
        'FooterRowCount': '0',
        'AppliedTableStyle': 'TableStyle/$ID/[Basic Table]',
        'TableDirection': 'LeftToRightDirection'
    }
    CELL_DEFAULTS = {
        'CellType': 'TextTypeCell',
        'AppliedCellStyle': 'CellStyle/$ID/[None]'
    }

    def __init__(self, self_id, element, inner_page_width, attributes=None, markup_tag=None):
        self._inner_page_width = inner_page_width
        super().__init__(self_id, element, attributes, markup_tag)

    def _add_story(self):
        # merge Story attributes
        story_attributes = self.merge_attributes(
            self.STORY_DEFAULTS,
            self._attributes.get('Story', {})
        )
        story_attributes.update({'Self': self.self_id})
        # Story
        story = etree.SubElement(
            self._etree,
            'Story',
            attrib=story_attributes
        )

        if self._markup_tag:
            # XMLElement to tag a story
            table_container = etree.SubElement(
                story,
                'XMLElement',
                attrib={
                    'Self': '{}_{}'.format(self.self_id, self._markup_tag.lower()),
                    'XMLContent': self.self_id,
                    'MarkupTag': 'XMLTag/{}'.format(self._markup_tag)
                }
            )
        else:
            table_container = story

        # create Table and insert it into table_container
        table_container.insert(1, self._create_table())
        # StoryPreference
        etree.SubElement(
            story,
            'StoryPreference',
            attrib=self.merge_attributes(
                self.STORYPREFERENCE_DEFAULTS,
                self._attributes.get('StoryPreference', {})
            )
        )

        return story

    def _create_table(self):
        table_data = {}
        table_data['cells'] = self._element.xpath('.//td')
        table_data['rows_count'] = int(self._element.xpath('count(.//tr)'))
        table_data['columns_count'] = int(self._element.xpath('count(.//td)') / table_data['rows_count'])
        # Table
        table = etree.Element(
            'Table',
            attrib=self.merge_attributes(
                self.TABLE_DEFAULTS,
                self._attributes.get('Table', {})
            )
        )
        table.set('Self', '{}_table'.format(self.self_id))
        table.set('BodyRowCount', str(table_data['rows_count']))
        table.set('ColumnCount', str(table_data['columns_count']))
        # Row(s)
        for i in range(table_data['rows_count']):
            etree.SubElement(
                table,
                'Row',
                attrib={
                    'Self': '{}_table_row{}'.format(self.self_id, i),
                    'Name': str(i)
                }
            )
        # Column(s)
        column_width = self._inner_page_width / table_data['columns_count']
        for i in range(table_data['columns_count']):
            etree.SubElement(
                table,
                'Column',
                attrib={
                    'Self': '{}_table_column{}'.format(self.self_id, i),
                    'Name': str(i),
                    'SingleColumnWidth': str(column_width)
                }
            )
        # Cells
        cell_counter = 0
        for r in range(table_data['rows_count']):
            for c in range(table_data['columns_count']):
                # Cell
                cell = etree.SubElement(
                    table,
                    'Cell',
                    attrib=self.merge_attributes(
                        self.CELL_DEFAULTS,
                        {
                            'Self': '{}_table_i{}'.format(self.self_id, cell_counter),
                            'Name': '{cell}:{row}'.format(cell=c, row=r),
                        }
                    )
                )
                # CharacterStyleRange(s) + <Br />
                for p in table_data['cells'][cell_counter].xpath('.//p'):
                    if cell.find('CharacterStyleRange') is not None:
                        etree.SubElement(
                            cell,
                            'Br'
                        )
                    cell[:] += self._handle_inline_tags(p)
                cell_counter += 1

        return table

    @property
    def length(self):
        raise NotImplementedError

    @staticmethod
    def guess_height(story, inner_width):
        table_height = 0
        table_element = story._etree.xpath('.//Table')[0]
        column_count = int(table_element.get('ColumnCount'))
        row_count = int(table_element.get('BodyRowCount'))

        for row_number in range(row_count):
            highest_cell_len = None

            for column_number in range(column_count):
                try:
                    cell = story._etree.xpath('.//Cell[@Name="{}:{}"]'.format(column_number, row_number))[0]
                except IndexError:
                    continue
                else:
                    current_cell_len = 0

                    for content in cell.xpath('.//Content'):
                        current_cell_len += len(" ".join(etree.XPath(".//text()")(content)).strip())

                    if not highest_cell_len or current_cell_len > highest_cell_len:
                        highest_cell_len = current_cell_len

            row_height = highest_cell_len / (inner_width / column_count) * 90 + 10

            if row_height < 20:
                row_height = 20
            table_height += row_height

        return table_height



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from functools import partial
from unittest.mock import MagicMock, patch
from flask import Flask
from .utils import mock_dictionaries
from superdesk.tests import TestCase
from superdesk.text_checkers import spellcheckers
from superdesk.text_checkers.spellcheckers import SPELLCHECKER_DEFAULT
from superdesk.text_checkers.spellcheckers.base import registered_spellcheckers
from superdesk.text_checkers.spellcheckers.default import Default
from superdesk import get_resource_service

spellcheckers.AUTO_IMPORT = False


MODEL = {
    "is": 1,
    "there": 1,
    "a": 1,
    "spelling": 1,
    "mistake": 1,
    "number": 1,
    "rendez-vous": 1,
    "this": 1,
    "are": 1,
    "proper": 1,
    "nouns": 1,
}


def load_spellcheckers():
    registered_spellcheckers.clear()
    app = Flask(__name__)
    spellcheckers.importSpellcheckers(app, spellcheckers.__name__)


class DefaultSpellcheckerTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_spellcheckers()

    def test_list(self):
        """Check that Default spellchecker is listed by spellcheckers_list service"""
        doc = {}
        spellcheckers_list = get_resource_service('spellcheckers_list')
        spellcheckers_list.on_fetched(doc)
        for checker in doc['spellcheckers']:
            if ((checker['name'] == SPELLCHECKER_DEFAULT
                 and 'label' in checker
                 and set(checker['capacities']) == set(Default.capacities)
                 and checker['languages'] == ['*'])):
                return
        self.fail("Defaut spellchecker not found")

    @patch('superdesk.get_resource_service', MagicMock(side_effect=partial(mock_dictionaries, model=MODEL)))
    def test_checker(self):
        """Check that spellchecking is working"""
        doc = {
            "spellchecker": "default",
            # the number is to test that they are not seen as mistakes
            # and the "rendez-vous" is to check that compound words work
            "text": "Is therre a speling mitake? A number: 123456, a rendez-vous",
            "suggestions": False,
            "language": "en",
            "use_internal_dict": False,
        }
        spellchecker = get_resource_service('spellchecker')
        spellchecker.create([doc])

        self.assertEqual(doc['errors'], [
            {'startOffset': 3, 'text': 'therre', 'type': 'spelling'},
            {'startOffset': 12, 'text': 'speling', 'type': 'spelling'},
            {'startOffset': 20, 'text': 'mitake', 'type': 'spelling'}])

    @patch('superdesk.get_resource_service', MagicMock(side_effect=partial(mock_dictionaries, model=MODEL)))
    def test_suggest(self):
        """Check that spelling suggestions are working"""

        doc = {
            "spellchecker": "default",
            "text": "mitake",
            "suggestions": True,
            "language": "en",
        }
        spellchecker = get_resource_service('spellchecker')
        spellchecker.create([doc])

        self.assertEqual(doc, {
            'language': 'en',
            'spellchecker': 'default',
            'suggestions': [{'text': 'mistake'}, {'text': 'Mistake'}],
            'text': 'mitake'})

    @patch('superdesk.get_resource_service', MagicMock(side_effect=partial(mock_dictionaries, model=MODEL)))
    def test_ignore(self):
        """Check that "ignore" is working (SDBELGA-165)"""
        doc = {
            "spellchecker": "default",
            "text": "This is are proper nouns: David, Jean, Arthur",
            "suggestions": False,
            "language": "en",
            "use_internal_dict": False,
        }
        spellchecker = get_resource_service('spellchecker')
        spellchecker.create([doc])

        self.assertEqual(doc['errors'], [
            {'startOffset': 26, 'text': 'David', 'type': 'spelling'},
            {'startOffset': 33, 'text': 'Jean', 'type': 'spelling'},
            {'startOffset': 39, 'text': 'Arthur', 'type': 'spelling'}
        ])

        # now the same check with ignore
        doc = doc.copy()
        doc['ignore'] = ["David", "Jean", "Arthur"]
        spellchecker.create([doc])

        self.assertEqual(doc['errors'], [])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from ..service import ProdApiService


class ItemsService(ProdApiService):
    excluded_fields = \
        {
            'fields_meta',
            'unique_id',
            'family_id',
            'event_id',
            'lock_session',
            'lock_action',
            'lock_time',
            'lock_user',
        } | ProdApiService.excluded_fields




import superdesk


def init_app(app):
    app.client_config.update({
        'tansa': {
            'base_url': app.config.get('TANSA_CLIENT_BASE_URL'),
            'app_id': app.config.get('TANSA_APP_ID'),
            'app_version': superdesk.__version__,
            'user_id': app.config.get('TANSA_USER_ID'),
            'profile_id': int(app.config['TANSA_PROFILE_ID']) if app.config.get('TANSA_PROFILE_ID') else None,
            'license_key': app.config.get('TANSA_LICENSE_KEY'),
            'profiles': app.config.get('TANSA_PROFILES'),
        },
    })



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import json
import flask
import logging
import superdesk
import multiprocessing
import werkzeug.exceptions

from flask import current_app as app
from eve.utils import date_to_str
from eve.versioning import insert_versioning_documents
from bson.objectid import ObjectId
from apps.archive.common import ITEM_OPERATION
from superdesk import get_resource_service
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.tests import clean_dbs, use_snapshot
from superdesk.utc import utcnow
from superdesk.timer import timer
from apps.search_providers import allowed_search_providers,\
    register_search_provider


logger = logging.getLogger(__name__)


def apply_placeholders(placeholders, text):
    if not placeholders or not text:
        return text
    for tag, value in placeholders.items():
        text = text.replace(tag, value)
    return text


def set_logged_user(username, password):
    auth_token = get_resource_service('auth').find_one(username=username, req=None)
    if not auth_token:
        user = {'username': username, 'password': password}
        get_resource_service('auth_db').post([user])
        auth_token = get_resource_service('auth').find_one(username=username, req=None)
    flask.g.user = get_resource_service('users').find_one(req=None, username=username)
    flask.g.auth = auth_token


def get_default_user():
    user = {'username': 'test_user', 'password': 'test_password', 'is_active': True, 'needs_activation': False,
            'first_name': 'first name', 'last_name': 'last name', 'email': 'test_user@test.com'}
    return user


def prepopulate_data(file_name, default_user=None, directory=None):
    if default_user is None:
        default_user = get_default_user()

    if not directory:
        directory = os.path.abspath(os.path.dirname(__file__))
    placeholders = {'NOW()': date_to_str(utcnow())}
    users = {default_user['username']: default_user['password']}
    default_username = default_user['username']
    file = os.path.join(directory, file_name)
    with open(file, 'rt', encoding='utf8') as app_prepopulation:
        json_data = json.load(app_prepopulation)
        for item in json_data:
            resource = item.get('resource', None)
            try:
                service = get_resource_service(resource)
            except KeyError:
                continue  # resource which is not configured - ignore
            username = item.get('username', None) or default_username
            set_logged_user(username, users[username])
            id_name = item.get('id_name', None)
            id_update = item.get('id_update', None)
            text = json.dumps(item.get('data', None))
            text = apply_placeholders(placeholders, text)
            data = json.loads(text)
            if resource:
                app.data.mongo._mongotize(data, resource)
            if resource == 'users':
                users.update({data['username']: data['password']})
            if id_update:
                id_update = apply_placeholders(placeholders, id_update)
                res = service.patch(ObjectId(id_update), data)
                if not res:
                    raise Exception()
            else:
                try:
                    ids = service.post([data])
                except werkzeug.exceptions.Conflict:
                    # instance was already prepopulated
                    break
                except superdesk.errors.SuperdeskApiError as e:
                    logger.exception(e)
                    continue  # an error raised by validation
                if not ids:
                    raise Exception()
                if id_name:
                    placeholders[id_name] = str(ids[0])

            if app.config['VERSION'] in data:
                number_of_versions_to_insert = data[app.config['VERSION']]
                doc_versions = []

                if data[ITEM_STATE] not in [CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED, CONTENT_STATE.KILLED]:
                    while number_of_versions_to_insert != 0:
                        doc_versions.append(data.copy())
                        number_of_versions_to_insert -= 1
                else:
                    if data[ITEM_STATE] in [CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED, CONTENT_STATE.CORRECTED]:
                        latest_version = data.copy()
                        doc_versions.append(latest_version)

                        published_version = data.copy()
                        published_version[ITEM_STATE] = CONTENT_STATE.PUBLISHED
                        published_version[ITEM_OPERATION] = 'publish'
                        published_version[app.config['VERSION']] = number_of_versions_to_insert - 1
                        doc_versions.append(published_version)

                        number_of_versions_to_insert -= 2
                    elif data[ITEM_STATE] == CONTENT_STATE.PUBLISHED:
                        published_version = data.copy()
                        doc_versions.append(published_version)
                        number_of_versions_to_insert -= 1

                    while number_of_versions_to_insert != 0:
                        doc = data.copy()
                        doc[ITEM_STATE] = CONTENT_STATE.PROGRESS
                        doc.pop(ITEM_OPERATION, '')
                        doc[app.config['VERSION']] = number_of_versions_to_insert
                        doc_versions.append(doc)

                        number_of_versions_to_insert -= 1

                insert_versioning_documents(resource, doc_versions if doc_versions else data)


prepopulate_schema = {
    'profile': {
        'type': 'string',
        'required': False,
        'default': 'app_prepopulate_data'
    },
    'remove_first': {
        'type': 'boolean',
        'required': False,
        'default': True
    },
}


class PrepopulateResource(Resource):
    """Prepopulate application data."""

    schema = prepopulate_schema
    resource_methods = ['POST']
    public_methods = ['POST']


class PrepopulateService(BaseService):
    def _create(self, docs):
        for doc in docs:
            if doc.get('remove_first'):
                clean_dbs(app, force=True)

            app.init_indexes()
            app.data.init_elastic(app)

            get_resource_service('users').stop_updating_stage_visibility()

            user = get_resource_service('users').find_one(username=get_default_user()['username'], req=None)
            if not user:
                get_resource_service('users').post([get_default_user()])

            prepopulate_data(doc.get('profile') + '.json', get_default_user())

            get_resource_service('users').start_updating_stage_visibility()
            get_resource_service('users').update_stage_visibility_for_users()

    def create(self, docs, **kwargs):
        with multiprocessing.Lock() as lock:
            with timer('prepopulate'):
                self._create(docs)
            if app.config.get('SUPERDESK_TESTING'):
                for provider in ['paimg', 'aapmm']:
                    if provider not in allowed_search_providers:
                        register_search_provider(provider, provider)
            return ['OK']


class AppPrepopulateCommand(superdesk.Command):
    """Prepopulate Superdesk using sample data.

    Useful for demo/development environment, but don't run in production,
    it's hard to get rid of such data later.

    Example:
    ::

        $ python manage.py app:prepopulate

    """

    option_list = [
        superdesk.Option('--file', '-f', dest='prepopulate_file', default='app_prepopulate_data.json'),
        superdesk.Option('--dir', '-d', dest='directory', default=None),
    ]

    def run(self, prepopulate_file, directory=None):
        user = get_resource_service('users').find_one(username=get_default_user()['username'], req=None)
        if not user:
            get_resource_service('users').post([get_default_user()])
        prepopulate_data(prepopulate_file, get_default_user(), directory)


superdesk.command('app:prepopulate', AppPrepopulateCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import bson
from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.commands.update_archived_document import UpdateArchivedDocumentCommand


class UpdateArchivedDocumentTestCase(TestCase):

    def setUp(self):
        self.guid = 'urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a'

        self.archived_only_data = [{
            "_id": bson.ObjectId("588c1b901d41c805dce70df0"),
            "guid": self.guid,
            "item_id": self.guid,
            "_current_version": 3,
            "type": "text",
            "abstract": "test",
            "state": "fetched",
            "slugline": "slugline",
            "headline": "headline",
            "flags": {"marked_archived_only": True},
            "subject": [{"qcode": "17004000", "name": "Statistics"}],
            "body_html": "Test Document body",
            'source': 'AAP',
            'linked_in_packages': [{
                "package": "urn:newsml:localhost:2017-01-28T15:28:33.535974:30c01757-cfb5-4985-a8f4-c3ecb253a244",
                "package_type": "takes"}]

        }, {
            "_id": bson.ObjectId("57d224de069b7f038e9d2a53"),
            "guid": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
            "item_id": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
            "_current_version": 4,
            "type": "text",
            "abstract": "test",
            "state": "fetched",
            "slugline": "slugline",
            "headline": "headline",
            "flags": {"marked_archived_only": True},
            "subject": [{"qcode": "17004000", "name": "Statistics"}],
            "body_html": "Test Document body",
            'source': 'AAP'
        }]

        self.archivedService = get_resource_service('archived')

    def test_update_source(self):
        self.archivedService.post(self.archived_only_data)
        UpdateArchivedDocumentCommand().run("['588c1b901d41c805dce70df0']", 'source', 'NTB')

        item = self.archivedService.get(req=None, lookup={'_id': '588c1b901d41c805dce70df0'})
        self.assertEqual('NTB', item[0].get('source'))

    def test_update_anpa_category_for_multiple_document(self):
        self.archivedService.post(self.archived_only_data)
        UpdateArchivedDocumentCommand().run("['588c1b901d41c805dce70df0', '57d224de069b7f038e9d2a53']",
                                            'anpa_category',
                                            '[{"scheme":null,"qcode":"f","subject":"04000000","name":"Finance"}]',
                                            True)

        item = self.archivedService.get(req=None, lookup={'_id': '588c1b901d41c805dce70df0'})
        self.assertEqual('f', item[0].get('anpa_category')[0].get('qcode'))
        item = self.archivedService.get(req=None, lookup={'_id': '57d224de069b7f038e9d2a53'})
        self.assertEqual('f', item[0].get('anpa_category')[0].get('qcode'))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .users import UsersResource
from .services import UsersService, DBUsersService, is_admin  # noqa
from superdesk.errors import SuperdeskApiError
import superdesk
import flask


def init_app(app):
    endpoint_name = 'users'
    service = DBUsersService(endpoint_name, backend=superdesk.get_backend())
    UsersResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name='users', label='User Management', description='User can manage users.')

    # Registering with intrinsic privileges because: A user should be allowed to update their own profile.
    superdesk.intrinsic_privilege(resource_name='users', method=['PATCH'])

    app.client_config.setdefault('user', {}).update({
        'username_pattern': app.config.get('USER_USERNAME_PATTERN'),
    })


def get_user_from_request(required=False):
    """
    Get user authenticated for current request.

    :param boolean required: if True and there is no user it will raise an error
    """

    user = flask.g.get('user', {})
    if '_id' not in user and required:
        raise SuperdeskApiError.notFoundError('Invalid user.')
    return user



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from datetime import timedelta

from flask import current_app as app
from superdesk.celery_task_utils import get_lock_id
from superdesk.lock import lock, unlock
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class RemoveExportedFiles(superdesk.Command):
    """Remove files from storage that were used for exporting items

    Example:
    ::

        $ python manage.py storage:remove_exported
        $ python manage.py storage:remove_exported --expire-hours=12

    """

    log_msg = ''
    expire_hours = 24

    option_list = [
        superdesk.Option('--expire-hours', '-e', dest='expire_hours', required=False, type=int)
    ]

    def run(self, expire_hours=None):
        if expire_hours:
            self.expire_hours = expire_hours
        elif 'TEMP_FILE_EXPIRY_HOURS' in app.config:
            self.expire_hours = app.config['TEMP_FILE_EXPIRY_HOURS']

        expire_at = utcnow() - timedelta(hours=self.expire_hours)
        self.log_msg = 'Expiry Time: {}.'.format(expire_at)
        logger.info('{} Starting to remove exported files from storage'.format(self.log_msg))

        lock_name = get_lock_id('storage', 'remove_exported')
        if not lock(lock_name, expire=300):
            logger.info('Remove exported files from storage task is already running')
            return

        try:
            logger.info('{} Removing expired temporary media files'.format(self.log_msg))
            self._remove_exported_files(expire_at)
        finally:
            unlock(lock_name)

        logger.info('{} Completed removing exported files from storage'.format(self.log_msg))

    def _remove_exported_files(self, expire_at):
        logger.info('{} Beginning to remove exported files from storage'.format(self.log_msg))
        for file_id in self._get_file_ids(expire_at):
            app.media.delete(file_id)

    def _get_file_ids(self, expire_at):
        files = app.media.find(folder='temp', upload_date={'$lte': expire_at})
        return [file['_id'] for file in files]


superdesk.command('storage:remove_exported', RemoveExportedFiles())



# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import time
import logging
from authlib.flask.oauth2 import AuthorizationServer
from authlib.oauth2.rfc6749 import grants
from authlib.jose import jwt
import superdesk
from .models import query_client, save_token


logger = logging.getLogger(__name__)

authorization = AuthorizationServer(
    query_client=query_client,
    save_token=save_token,
)


bp = superdesk.Blueprint('auth_server', __name__)

TOKEN_ENDPOINT = '/auth_server/token'
shared_secret = None
expiration_delay = 0


@bp.route(TOKEN_ENDPOINT, methods=['POST'])
def issue_token():
    return authorization.create_token_response()


def generate_jwt_token(client, grant_type, user, scope):
    header = {"alg": "HS256"}
    payload = {
        "iss": "Superdesk Auth Server",
        "iat": int(time.time()),
        "exp": int(time.time() + expiration_delay),
        "client_id": client.client_id,
        "scope": client.scope,
    }
    return jwt.encode(header, payload, shared_secret)


def config_oauth(app):
    global expiration_delay
    expiration_delay = app.config['AUTH_SERVER_EXPIRATION_DELAY']

    global shared_secret
    shared_secret = app.config['AUTH_SERVER_SHARED_SECRET']
    if not shared_secret.strip():
        logger.warning(
            "No shared secret set, please set it using AUTH_SERVER_SHARED_SECRET "
            "environment variable or setting. Authorisation server can't be used")
        return

    app.config['OAUTH2_ACCESS_TOKEN_GENERATOR'] = generate_jwt_token
    app.config['OAUTH2_TOKEN_EXPIRES_IN'] = {'client_credentials': expiration_delay}
    authorization.init_app(app)
    authorization.register_grant(grants.ClientCredentialsGrant)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import flask

from superdesk import get_backend
from superdesk.tests import TestCase
from superdesk.users.services import UsersService, compare_preferences


class PrivilegesTestCase(TestCase):

    def setUp(self):
        with self.app.app_context():
            self.service = UsersService('users', backend=get_backend())

    def test_admin_has_all_privileges(self):
        with self.app.app_context():
            user = {'user_type': 'administrator'}
            self.service.set_privileges(user, None)
            self.assertEqual(user['active_privileges']['users'], 1)

    def test_user_has_merged_privileges(self):
        with self.app.app_context():
            user = {'user_type': 'user', 'privileges': {'users': 1}}
            role = {'privileges': {'archive': 1}}
            self.service.set_privileges(user, role)
            self.assertEqual(user['active_privileges']['users'], 1)
            self.assertEqual(user['active_privileges']['archive'], 1)

    def test_user_with_privilege_can_change_his_role(self):
        with self.app.app_context():
            flask.g.user = {'user_type': 'administrator'}
            ids = self.service.create([{'name': 'user', 'user_type': 'administrator'}])
            doc_old = self.service.find_one(None, _id=ids[0])
            self.service.update(ids[0], {'role': '1'}, doc_old)
            self.assertIsNotNone(self.service.find_one(req=None, role='1'))

    def test_compare_preferences(self):
        original_preferences = {
            "unlock": 1,
            "archive": 1,
            "spike": 1,
            "unspike": 1,
            "ingest_move": 0
        }

        updated_preferences = {
            "unlock": 0,
            "archive": 1,
            "spike": 1,
            "ingest": 1,
            "ingest_move": 1,
        }

        added, removed, modified = compare_preferences(original_preferences, updated_preferences)
        self.assertEqual(1, len(added))
        self.assertEqual(1, len(removed))
        self.assertEqual(2, len(modified))
        self.assertTrue((1, 0) in modified.values())
        self.assertTrue((0, 1) in modified.values())




import io
import flask
import unittest
import requests_mock

from superdesk.media.media_operations import download_file_from_url


class MediaOperationsTestCase(unittest.TestCase):

    def test_download_file_from_url_relative(self):
        app = flask.Flask(__name__)
        app.config['SERVER_NAME'] = 'localhost'
        body = io.BytesIO(b'data')
        with app.app_context():
            with requests_mock.mock() as mock:
                mock.get('http://localhost/test/foo.jpg', body=body)
                out = download_file_from_url('/test/foo.jpg')
        self.assertEqual(b'data', out[0].read())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from .resource import ConceptItemsResource, CONCEPT_ITEMS_PRIVELEGE
from .service import ConceptItemsService


def init_app(app):
    endpoint_name = 'concept_items'
    service = ConceptItemsService(endpoint_name, backend=superdesk.get_backend())
    ConceptItemsResource(endpoint_name, app=app, service=service)

    superdesk.privilege(
        name=CONCEPT_ITEMS_PRIVELEGE,
        label='Concept items management',
        description='User can manage concept items.'
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .volume_cubic_feet_to_metric import cubic_feet_to_metric


class VolumeTestCase(unittest.TestCase):

    def test(self):
        text = 'Total volume is 100.50 cubic feet for this land. '\
               'Total volume is 15.7 cubic ft for this land. '\
               'Total volume is 1 Cubic Foot for this land. '\
               'Total volume is 1-16 cu-ft for this land. '\
               'Total volume is 1-16 cb. ft for this land. '\
               'Total volume is 16.7-Cubic-ft for this land. '\
               'Total volume is 16,500-cu. ft. for this land. '\

        item = {'body_html': text}
        res, diff = cubic_feet_to_metric(item)
        self.assertEqual(diff['100.50 cubic feet'], '100.50 cubic feet (2.85 cubic meter)')
        self.assertEqual(diff['15.7 cubic ft'], '15.7 cubic ft (0.4 cubic meter)')
        self.assertEqual(diff['1 Cubic Foot'], '1 Cubic Foot (0.03 cubic meter)')
        self.assertEqual(diff['1-16 cu-ft'], '1-16 cu-ft (0.03-0.45 cubic meter)')
        self.assertEqual(diff['1-16 cb. ft'], '1-16 cb. ft (0.03-0.45 cubic meter)')
        self.assertEqual(diff['16.7-Cubic-ft'], '16.7-Cubic-ft (0.5 cubic meter)')
        self.assertEqual(diff['16,500-cu. ft'], '16,500-cu. ft (467 cubic meter)')
        self.assertEqual(res['body_html'], item['body_html'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase
from apps.templates.content_templates import get_item_from_template, render_content_template


class RenderTemplateTestCase(TestCase):

    def test_render_content_template(self):
        template = {
            '_id': 'foo',
            'template_name': 'test',
            'template_desks': ['sports'],
            'data': {
                'headline': 'Foo Template: {{item.headline}}',
                'body_html': 'This article has slugline: {{item.slugline}} and dateline: {{item.dateline["text"]}} '
                             'at {{item.versioncreated | format_datetime("Australia/Sydney", "%d %b %Y %H:%S %Z")}}',
                'urgency': 1, 'priority': 3,
                'dateline': {},
                'anpa_take_key': 'this is test',
                'place': ['Australia']
            }
        }

        item = {
            '_id': '123', 'headline': 'Test Template',
            'slugline': 'Testing', 'body_html': 'This is test story',
            'dateline': {
                'text': 'hello world'
            },
            'urgency': 4, 'priority': 6,
            'versioncreated': '2015-06-01T22:54:53+0000',
            'place': ['NSW']
        }

        updates = render_content_template(item, template)
        self.assertEqual(updates['headline'], 'Foo Template: Test Template')
        self.assertEqual(updates['urgency'], 1)
        self.assertEqual(updates['priority'], 3)
        self.assertEqual(updates['body_html'], 'This article has slugline: Testing and dateline: '
                                               'hello world at 02 Jun 2015 08:53 AEST')
        self.assertListEqual(updates['place'], ['Australia'])

    def test_headline_strip_tags(self):
        template = {'data': {'headline': ' test\nit<br>'}}

        updates = render_content_template({}, template)
        self.assertEqual('test it', updates['headline'])

        item = get_item_from_template(template)
        self.assertEqual('test it', item['headline'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.errors import ParserError
from superdesk.metadata.item import CONTENT_TYPE
from superdesk import etree as sd_etree
from superdesk import text_utils
from superdesk.utc import local_to_utc
import logging

logger = logging.getLogger(__name__)
IPTC_NS = 'http://iptc.org/std/nar/2006-10-01/'
STT_LOCATION_MAP = {
    "sttcity": {"qcode": "locality_code", "name": "locality"},
    "sttstate": {"qcode": "state_code", "name": "state"},
    "sttcountry": {"qcode": "country_code", "name": "country"},
    "wldreg": {"qcode": "world_region_code", "name": "world_region"},
}

TIMEZONE = 'Europe/Helsinki'


class STTNewsMLFeedParser(NewsMLTwoFeedParser):
    """
    Feed Parser which can parse STT variant of NewsML
    """

    NAME = 'sttnewsml'
    label = "STT NewsML"
    SUBJ_QCODE_PREFIXES = {
        'stt-subj': None,
        'sttdepartment': 'sttdepartment',
        'sttsubj': 'sttsubj',
        'sttdone1': 'sttdone1',
    }

    def can_parse(self, xml):
        return xml.tag.endswith('newsItem')

    def parse(self, xml, provider=None):
        self.root = xml
        try:
            item = self.parse_item(xml)
            if not item.get('headline'):
                item['headline'] = text_utils.get_text(item.get('body_html', '') or '', 'html')[:100]

            self.parse_version_dates(xml, item)

            # abstract
            try:
                abstract = xml.xpath("//iptc:description[@role='drol:summary']", namespaces={'iptc': IPTC_NS})[0].text
            except IndexError:
                pass
            else:
                if abstract:
                    item['abstract'] = abstract

            # genre
            for genre_elt in xml.xpath("//iptc:genre", namespaces={'iptc': IPTC_NS}):
                qcode = genre_elt.get('qcode')
                if qcode is None:
                    continue
                elif qcode.startswith('sttgenre:'):
                    qcode = qcode[9:]
                    genre_data = {'qcode': qcode}
                    name_elt = genre_elt.find(self.qname('name'))
                    name = name_elt.text if name_elt is not None and name_elt.text else ""
                    try:
                        name = self.getVocabulary("genre", qcode, name)
                    except ValueError:
                        continue
                    else:
                        genre_data['name'] = name
                        item.setdefault('genre', []).append(genre_data)
                elif qcode.startswith('sttversion:'):
                    qcode = qcode[11:]
                    version_data = {'qcode': qcode, 'scheme': 'sttversion'}
                    name_elt = genre_elt.find(self.qname('name'))
                    name = name_elt.text if name_elt is not None and name_elt.text else ""
                    try:
                        name = self.getVocabulary("sttgenre", qcode, name)
                    except ValueError:
                        continue
                    else:
                        version_data['name'] = name
                        item.setdefault('subject', []).append(version_data)

            # location
            for location_elt in xml.xpath("//iptc:assert", namespaces={'iptc': IPTC_NS}):
                qcode = location_elt.get("qcode")
                if not qcode or not qcode.startswith("sttlocmeta:"):
                    continue
                qcode = qcode.split(':')[-1]
                location_data = {"scheme": "sttlocmeta", "qcode": qcode}
                location_name = location_elt.find(self.qname('name'))
                if location_name is not None:
                    location_data['name'] = location_name.text
                for broader_elt in location_elt.xpath(".//iptc:broader[@type='cpnat:geoArea']",
                                                      namespaces={'iptc': IPTC_NS}):
                    qcode = broader_elt.get('qcode')
                    if not qcode:
                        continue
                    for key, mapping in STT_LOCATION_MAP.items():
                        if qcode.startswith(key + ":"):
                            if "qcode" in mapping:
                                qcode = qcode[len(key) + 1:]
                            try:
                                name = broader_elt.find(self.qname('name')).text
                            except AttributeError:
                                name = ""
                            try:
                                name = self.getVocabulary(key, qcode, name)
                            except ValueError:
                                continue
                            else:
                                location_data[mapping["qcode"]] = qcode
                                if "name" in mapping:
                                    location_data[mapping["name"]] = name
                item.setdefault('place', []).append(location_data)

            # public editorial note
            if 'ednote' in item:
                # stt has specific roles for public and private editorial notes
                # so we remove ednote found by parent parser, as it takes first one
                # as a public note
                del item['ednote']
            try:
                ednote = xml.xpath("//iptc:edNote[@role='sttnote:public']", namespaces={'iptc': IPTC_NS})[0].text
            except IndexError:
                pass
            else:
                if ednote:
                    item['ednote'] = ednote

            # private editorial note
            try:
                private_note = xml.xpath("//iptc:edNote[@role='sttnote:private']", namespaces={'iptc': IPTC_NS})[0].text
            except IndexError:
                pass
            else:
                if private_note:
                    item.setdefault('extra', {})['sttnote_private'] = private_note

            return [item]
        except Exception as ex:
            raise ParserError.newsmlTwoParserError(ex, provider)

    def parse_inline_content(self, tree, item):
        html_elt = tree.find(self.qname('html'))
        body_elt = html_elt.find(self.qname('body'))
        body_elt = sd_etree.clean_html(body_elt)

        content = dict()
        content['contenttype'] = tree.attrib['contenttype']
        if len(body_elt) > 0:
            contents = [sd_etree.to_string(e, encoding='unicode', method="html") for e in body_elt]
            content['content'] = '\n'.join(contents)
        elif body_elt.text:
            content['content'] = '<pre>' + body_elt.text + '</pre>'
            content['format'] = CONTENT_TYPE.PREFORMATTED

        if content.get('content'):
            content['content'] = content['content'].replace('&lt;endash&gt;-&lt;/endash&gt;', '-')

        return content

    def datetime(self, value):
        """When there is no timezone info, assume it's Helsinki timezone."""
        parsed = super().datetime(value)
        if '+' not in value:
            return local_to_utc(TIMEZONE, parsed)
        return parsed

    def parse_content_meta(self, tree, item):
        meta = super().parse_content_meta(tree, item)
        creditline = meta.find(self.qname('creditline'))
        if creditline is not None:
            item['source'] = creditline.text.replace('–', '-').rstrip('-')  # replace endash with dash

    def parse_version_dates(self, tree, item):
        """
        Uses contentModified field if exists to set versioncreated for item
        If not it uses to versioncreated
        https://dev.sourcefabric.org/browse/STTNHUB-84
        """

        meta = tree.find(self.qname('contentMeta'))

        content_updated = meta.find(self.qname('contentModified'))
        if content_updated is not None and content_updated.text:
            item['versioncreated'] = self.datetime(content_updated.text)

        content_created = meta.find(self.qname('contentCreated'))
        if content_created is not None and content_created.text and not item.get('versioncreated'):
            item['versioncreated'] = self.datetime(content_created.text)

        if not item.get('versioncreated'):
            raise Exception('contentModified and contentCreated fields are empty in ingest file')

        item.setdefault('firstpublished', item.get('versioncreated'))


register_feed_parser(STTNewsMLFeedParser.NAME, STTNewsMLFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import etree as sd_etree
from lxml import etree
from flask_babel import lazy_gettext


def extract_html_macro(item, **kwargs):
    """Delete from body_html all html tags except links"""
    if 'body_html' not in item:
        return None

    root = sd_etree.parse_html(item['body_html'], content='html')

    links = {}
    count = 0
    # extract all links and add them to a dictionary with a unique
    # generated key for every link
    for a in root.findall('.//a'):
        links['__##link' + str(count) + '##__'] = etree.tostring(a, encoding="unicode")
        count = count + 1

    # replace all text links with the generated keys
    # regenerate html back from root in order to avoid issues
    # on link replacements where are used text links generated from root
    body_html = etree.tostring(root, encoding="unicode")
    for link in links:
        body_html = body_html.replace(links[link], link)
    body_html = body_html.replace('<p>', '__##br##__')
    body_html = body_html.replace('</p>', '__##br##__')
    body_html = body_html.replace('<br/>', '__##br##__')

    # extract text from the html that don't contains any link,
    # it just contains link keys that are not affected by text extraction
    # because they are already text
    root = sd_etree.parse_html(body_html, content='html')
    body_html = etree.tostring(root, encoding="unicode", method="text")

    # in extracted text replace the link keys with links
    for link in links:
        body_html = body_html.replace(link, links[link])

    body_html = body_html.replace('\n', '__##br##__')
    list_paragraph = body_html.split('__##br##__')
    item['body_html'] = ''.join('<p>' + p + '</p>' for p in list_paragraph if p and p.strip())
    return item


name = 'Extract Html Macro'
label = lazy_gettext('Extract Html Macro')
callback = extract_html_macro
access_type = 'frontend'
action_type = 'direct'
